@@ -34,7 +34,7 @@ function QueueListModel(parent) {
     // External var's
     self.queueItems = ko.observableArray([]);
     self.totalItems = ko.observable(0);
-    self.isMultiEditing = ko.observable(false);
+    self.isMultiEditing = ko.observable(false).extend({ persist: 'queueIsMultiEditing' });
     self.isLoading = ko.observable(false).extend({ rateLimit: 100 });
     self.multiEditItems = ko.observableArray([]);
     self.categoriesList = ko.observableArray([]);
@@ -228,11 +228,11 @@ function QueueListModel(parent) {
         switch($(event.currentTarget).data('action')) {
             case 'sortAgeAsc':
                 sort = 'avg_age';
-                dir = 'asc';
+                dir = 'desc';
                 break;
             case 'sortAgeDesc':
                 sort = 'avg_age';
-                dir = 'desc';
+                dir = 'asc';
                 break;
             case 'sortNameAsc':
                 sort = 'name';
@@ -440,12 +440,11 @@ function QueueListModel(parent) {
                 if(response.status) {
                     // Make sure the queue doesnt flicker and then fade-out
                     self.isLoading(true)
-                    $('.delete input:checked').parents('tr').fadeOut(fadeOnDeleteDuration)
                     self.parent.refresh()
                     // Empty it
                     self.multiEditItems.removeAll();
                     // Hide notification
-                    hideNotification(true)
+                    hideNotification()
                 }
             })
         }
@@ -471,6 +470,7 @@ function QueueListModel(parent) {
 function QueueModel(parent, data) {
     var self = this;
     self.parent = parent;
+    self.rawLabels = []
 
     // Job info
     self.id = data.nzo_id;
@@ -478,6 +478,7 @@ function QueueModel(parent, data) {
     self.password = ko.observable(data.password);
     self.index = ko.observable(data.index);
     self.status = ko.observable(data.status);
+    self.labels = ko.observableArray(data.labels);
     self.isGrabbing = ko.observable(data.status == 'Grabbing' || data.avg_age == '-')
     self.isFetchingBlocks = data.status == 'Fetching' || data.priority == 'Repair' // No need to update
     self.totalMB = ko.observable(parseFloat(data.mb));
@@ -530,11 +531,10 @@ function QueueModel(parent, data) {
         return self.name()
     })
     self.missingText = ko.pureComputed(function() {
-        // Check for missing data, the value is arbitrary! (1%)
-        if(self.missingMB()/self.totalMB() > 0.01) {
+        // Check for missing data, can show 0 if article-size is smaller than 500K, but we accept that
+        if(self.missingMB()) {
             return self.missingMB().toFixed(0) + ' MB ' + glitterTranslate.misingArt
         }
-        return;
     })
     self.statusText = ko.computed(function() {
         // Checking
@@ -565,10 +565,9 @@ function QueueModel(parent, data) {
         return 'glyphicon-pause'
     })
 
-    // Extra queue column
-    self.extraText = ko.pureComputed(function() {
-        // Picked anything?
-        switch(self.parent.parent.extraQueueColumn()) {
+    // Extra queue columns
+    self.showColumn = function(param) {
+        switch(param) {
             case 'category':
                 // Exception for *
                 if(self.category() == "*")
@@ -588,7 +587,7 @@ function QueueModel(parent, data) {
                 return self.avg_age();
         }
         return;
-    })
+    };
 
     // Every update
     self.updateFromData = function(data) {
@@ -610,6 +609,14 @@ function QueueModel(parent, data) {
         self.unpackopts(parseInt(data.unpackopts)) // UnpackOpts fails if not parseInt'd!
         self.pausedStatus(data.status == 'Paused');
         self.timeLeft(data.timeleft);
+
+        // Did the label-list change?
+        // Otherwise KO will send updates to all texts during refresh()
+        if(self.rawLabels != data.labels.toString()) {
+            // Update
+            self.labels(data.labels);
+            self.rawLabels = data.labels.toString();
+        }
     };
 
     // Pause individual download
@@ -644,6 +651,9 @@ function QueueModel(parent, data) {
         // Anything change or empty?
         if(!newName || self.name() == newName) return;
 
+        // Rename would abort Direct Unpack, so ask if user is sure
+        if(self.direct_unpack() && !confirm(glitterTranslate.renameAbort)) return;
+
         // Send rename
         callAPI({
                 mode: 'queue',
@@ -716,7 +726,7 @@ function QueueModel(parent, data) {
     // Remove 1 download from queue
     self.removeDownload = function(item, event) {
         // Confirm and remove
-        if(!self.parent.parent.confirmDeleteQueue() || confirm(glitterTranslate.removeDow1)) {
+        if(!self.parent.parent.confirmDeleteQueue() || confirm(glitterTranslate.deleteMsg + ":\n" + item.name() + "\n\n" + glitterTranslate.removeDow1)) {
             var itemToDelete = this;
 
             // Show notification
@@ -728,17 +738,14 @@ function QueueModel(parent, data) {
                 del_files: 1,
                 value: item.id
             }).then(function(response) {
-                // Fade and remove
-                $(event.currentTarget).parent().parent().fadeOut(fadeOnDeleteDuration, function() {
-                    // Make sure no flickering (if there are more items left) and then remove
-                    self.parent.isLoading(self.parent.totalItems() > 1)
-                    parent.queueItems.remove(itemToDelete);
-                    parent.multiEditItems.remove(function(inList) { return inList.id == itemToDelete.id; })
-                    self.parent.parent.refresh();
-                    // Hide notifcation
-                    hideNotification(true)
-                })
+                // Make sure no flickering (if there are more items left) and then remove
+                self.parent.isLoading(self.parent.totalItems() > 1)
+                parent.queueItems.remove(itemToDelete);
+                parent.multiEditItems.remove(function(inList) { return inList.id == itemToDelete.id; })
+                self.parent.parent.refresh();
+                // Hide notifcation
+                hideNotification()
             });
         }
     };
-}
\ No newline at end of file
+}
