@@ -1,5 +1,5 @@
 #!/usr/bin/python3 -OO
-# Copyright 2007-2020 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2022 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -24,74 +24,167 @@ import queue
 import logging
 import re
 from threading import Thread
-from time import sleep
 import hashlib
+import ctypes
+from typing import Tuple, Optional, List
 
 import sabnzbd
-from sabnzbd.misc import get_all_passwords
-from sabnzbd.filesystem import set_permissions, clip_path, has_win_device, diskspace, get_filename, get_ext
+from sabnzbd.misc import get_all_passwords, match_str
+from sabnzbd.filesystem import (
+    set_permissions,
+    clip_path,
+    has_win_device,
+    diskspace,
+    get_filename,
+    has_unwanted_extension,
+)
 from sabnzbd.constants import Status, GIGI, MAX_ASSEMBLER_QUEUE
 import sabnzbd.cfg as cfg
-from sabnzbd.articlecache import ArticleCache
-from sabnzbd.postproc import PostProcessor
-import sabnzbd.downloader
+from sabnzbd.nzbstuff import NzbObject, NzbFile
 import sabnzbd.par2file as par2file
 import sabnzbd.utils.rarfile as rarfile
-from sabnzbd.rating import Rating
 
 
 class Assembler(Thread):
-    do = None  # Link to the instance of this method
-
     def __init__(self):
-        Thread.__init__(self)
-        self.queue = queue.Queue()
-        Assembler.do = self
+        super().__init__()
+        self.queue: queue.Queue[Tuple[Optional[NzbObject], Optional[NzbFile], Optional[bool]]] = queue.Queue()
 
     def stop(self):
-        self.process(None)
+        self.queue.put((None, None, None))
 
-    def process(self, job):
-        self.queue.put(job)
+    def process(self, nzo: NzbObject, nzf: Optional[NzbFile] = None, file_done: Optional[bool] = None):
+        self.queue.put((nzo, nzf, file_done))
 
     def queue_full(self):
         return self.queue.qsize() >= MAX_ASSEMBLER_QUEUE
 
     def run(self):
         while 1:
-            job = self.queue.get()
-            if not job:
+            # Set NzbObject and NzbFile objects to None so references
+            # from this thread do not keep the objects alive (see #1628)
+            nzo = nzf = None
+            nzo, nzf, file_done = self.queue.get()
+            if not nzo:
                 logging.info("Shutting down")
                 break
 
-            nzo, nzf, file_done = job
-
             if nzf:
                 # Check if enough disk space is free after each file is done
                 # If not enough space left, pause downloader and send email
-                if (
-                    file_done
-                    and diskspace(force=True)["download_dir"][1] < (cfg.download_free.get_float() + nzf.bytes) / GIGI
-                ):
-                    # Only warn and email once
-                    if not sabnzbd.downloader.Downloader.do.paused:
+                if file_done and not sabnzbd.Downloader.paused:
+                    freespace = diskspace(force=True)
+                    full_dir = None
+                    required_space = (cfg.download_free.get_float() + nzf.bytes) / GIGI
+                    if freespace["download_dir"][1] < required_space:
+                        full_dir = "download_dir"
+
+                    # Enough space in download_dir, check complete_dir
+                    complete_free = cfg.complete_free.get_float()
+                    if complete_free > 0 and not full_dir:
+                        required_space = 0
+                        if cfg.direct_unpack():
+                            required_space = (complete_free + nzo.bytes_downloaded) / GIGI
+                        else:
+                            # Continue downloading until 95% complete before checking
+                            if nzo.bytes_tried > (nzo.bytes - nzo.bytes_par2) * 0.95:
+                                required_space = (complete_free + nzo.bytes) / GIGI
+
+                        if required_space and freespace["complete_dir"][1] < required_space:
+                            full_dir = "complete_dir"
+
+                    if full_dir:
                         logging.warning(T("Too little diskspace forcing PAUSE"))
                         # Pause downloader, but don't save, since the disk is almost full!
-                        sabnzbd.downloader.Downloader.do.pause()
+                        sabnzbd.Downloader.pause()
+                        if cfg.fulldisk_autoresume():
+                            sabnzbd.Scheduler.plan_diskspace_resume(full_dir, required_space)
                         sabnzbd.emailer.diskfull_mail()
-                        # Abort all direct unpackers, just to be sure
-                        sabnzbd.directunpacker.abort_all()
 
                 # Prepare filepath
                 filepath = nzf.prepare_filepath()
 
                 if filepath:
-                    logging.debug("Decoding part of %s", filepath)
                     try:
+                        logging.debug("Decoding part of %s", filepath)
                         self.assemble(nzf, file_done)
+
+                        # Continue after partly written data
+                        if not file_done:
+                            continue
+
+                        # Clean-up admin data
+                        logging.info("Decoding finished %s", filepath)
+                        nzf.remove_admin()
+
+                        # Do rar-related processing
+                        if rarfile.is_rarfile(filepath):
+                            # Encryption and unwanted extension detection
+                            rar_encrypted, unwanted_file = check_encrypted_and_unwanted_files(nzo, filepath)
+                            if rar_encrypted:
+                                if cfg.pause_on_pwrar() == 1:
+                                    logging.warning(
+                                        T(
+                                            'Paused job "%s" because of encrypted RAR file (if supplied, all passwords were tried)'
+                                        ),
+                                        nzo.final_name,
+                                    )
+                                    nzo.pause()
+                                else:
+                                    logging.warning(
+                                        T(
+                                            'Aborted job "%s" because of encrypted RAR file (if supplied, all passwords were tried)'
+                                        ),
+                                        nzo.final_name,
+                                    )
+                                    nzo.fail_msg = T("Aborted, encryption detected")
+                                    sabnzbd.NzbQueue.end_job(nzo)
+
+                            if unwanted_file:
+                                # Don't repeat the warning after a user override of an unwanted extension pause
+                                if nzo.unwanted_ext == 0:
+                                    logging.warning(
+                                        T('In "%s" unwanted extension in RAR file. Unwanted file is %s '),
+                                        nzo.final_name,
+                                        unwanted_file,
+                                    )
+                                logging.debug(T("Unwanted extension is in rar file %s"), filepath)
+                                if cfg.action_on_unwanted_extensions() == 1 and nzo.unwanted_ext == 0:
+                                    logging.debug("Unwanted extension ... pausing")
+                                    nzo.unwanted_ext = 1
+                                    nzo.pause()
+                                if cfg.action_on_unwanted_extensions() == 2:
+                                    logging.debug("Unwanted extension ... aborting")
+                                    nzo.fail_msg = T("Aborted, unwanted extension detected")
+                                    sabnzbd.NzbQueue.end_job(nzo)
+
+                            # Add to direct unpack
+                            nzo.add_to_direct_unpacker(nzf)
+
+                        elif par2file.is_parfile(filepath):
+                            # Parse par2 files, cloaked or not
+                            nzo.handle_par2(nzf, filepath)
+
+                        filter_output, reason = nzo_filtered_by_rating(nzo)
+                        if filter_output == 1:
+                            logging.warning(
+                                T('Paused job "%s" because of rating (%s)'),
+                                nzo.final_name,
+                                reason,
+                            )
+                            nzo.pause()
+                        elif filter_output == 2:
+                            logging.warning(
+                                T('Aborted job "%s" because of rating (%s)'),
+                                nzo.final_name,
+                                reason,
+                            )
+                            nzo.fail_msg = T("Aborted, rating filter matched (%s)") % reason
+                            sabnzbd.NzbQueue.end_job(nzo)
+
                     except IOError as err:
-                        # If job was deleted or in active post-processing, ignore error
-                        if not nzo.deleted and not nzo.is_gone() and not nzo.pp_active:
+                        # If job was deleted/finished or in active post-processing, ignore error
+                        if not nzo.pp_or_finished:
                             # 28 == disk full => pause downloader
                             if err.errno == 28:
                                 logging.error(T("Disk full! Forcing Pause"))
@@ -99,90 +192,21 @@ class Assembler(Thread):
                                 logging.error(T("Disk error on creating file %s"), clip_path(filepath))
                             # Log traceback
                             logging.info("Traceback: ", exc_info=True)
+                            if sabnzbd.WIN32:
+                                logging.info("Winerror: %s", hex(ctypes.windll.ntdll.RtlGetLastNtStatus() + 2**32))
                             # Pause without saving
-                            sabnzbd.downloader.Downloader.do.pause()
-                        continue
+                            sabnzbd.Downloader.pause()
+                        else:
+                            logging.debug("Ignoring error %s for %s, already finished or in post-proc", err, filepath)
                     except:
                         logging.error(T("Fatal error in Assembler"), exc_info=True)
                         break
-
-                    # Continue after partly written data
-                    if not file_done:
-                        continue
-
-                    # Clean-up admin data
-                    logging.info("Decoding finished %s", filepath)
-                    nzf.remove_admin()
-
-                    # Do rar-related processing
-                    if rarfile.is_rarfile(filepath):
-                        # Encryption and unwanted extension detection
-                        rar_encrypted, unwanted_file = check_encrypted_and_unwanted_files(nzo, filepath)
-                        if rar_encrypted:
-                            if cfg.pause_on_pwrar() == 1:
-                                logging.warning(
-                                    T(
-                                        'Paused job "%s" because of encrypted RAR file (if supplied, all passwords were tried)'
-                                    ),
-                                    nzo.final_name,
-                                )
-                                nzo.pause()
-                            else:
-                                logging.warning(
-                                    T(
-                                        'Aborted job "%s" because of encrypted RAR file (if supplied, all passwords were tried)'
-                                    ),
-                                    nzo.final_name,
-                                )
-                                nzo.fail_msg = T("Aborted, encryption detected")
-                                sabnzbd.nzbqueue.NzbQueue.do.end_job(nzo)
-
-                        if unwanted_file:
-                            logging.warning(
-                                T('In "%s" unwanted extension in RAR file. Unwanted file is %s '),
-                                nzo.final_name,
-                                unwanted_file,
-                            )
-                            logging.debug(T("Unwanted extension is in rar file %s"), filepath)
-                            if cfg.action_on_unwanted_extensions() == 1 and nzo.unwanted_ext == 0:
-                                logging.debug("Unwanted extension ... pausing")
-                                nzo.unwanted_ext = 1
-                                nzo.pause()
-                            if cfg.action_on_unwanted_extensions() == 2:
-                                logging.debug("Unwanted extension ... aborting")
-                                nzo.fail_msg = T("Aborted, unwanted extension detected")
-                                sabnzbd.nzbqueue.NzbQueue.do.end_job(nzo)
-
-                        # Add to direct unpack
-                        nzo.add_to_direct_unpacker(nzf)
-
-                    elif par2file.is_parfile(filepath):
-                        # Parse par2 files, cloaked or not
-                        nzo.handle_par2(nzf, filepath)
-
-                    filter_output, reason = nzo_filtered_by_rating(nzo)
-                    if filter_output == 1:
-                        logging.warning(
-                            T('Paused job "%s" because of rating (%s)'),
-                            nzo.final_name,
-                            reason,
-                        )
-                        nzo.pause()
-                    elif filter_output == 2:
-                        logging.warning(
-                            T('Aborted job "%s" because of rating (%s)'),
-                            nzo.final_name,
-                            reason,
-                        )
-                        nzo.fail_msg = T("Aborted, rating filter matched (%s)") % reason
-                        sabnzbd.nzbqueue.NzbQueue.do.end_job(nzo)
-
             else:
-                sabnzbd.nzbqueue.NzbQueue.do.remove(nzo.nzo_id, add_to_history=False, cleanup=False)
-                PostProcessor.do.process(nzo)
+                sabnzbd.NzbQueue.remove(nzo.nzo_id, cleanup=False)
+                sabnzbd.PostProcessor.process(nzo)
 
     @staticmethod
-    def assemble(nzf, file_done):
+    def assemble(nzf: NzbFile, file_done: bool):
         """Assemble a NZF from its table of articles
         1) Partial write: write what we have
         2) Nothing written before: write all
@@ -191,7 +215,8 @@ class Assembler(Thread):
         if not nzf.md5:
             nzf.md5 = hashlib.md5()
 
-        with open(nzf.filepath, "ab") as fout:
+        # We write large article-sized chunks, so we can safely skip the buffering of Python
+        with open(nzf.filepath, "ab", buffering=0) as fout:
             for article in nzf.decodetable:
                 # Break if deleted during writing
                 if nzf.nzo.status is Status.DELETED:
@@ -203,7 +228,7 @@ class Assembler(Thread):
 
                 # Write all decoded articles
                 if article.decoded:
-                    data = ArticleCache.do.load_article(article)
+                    data = sabnzbd.ArticleCache.load_article(article)
                     # Could be empty in case nzo was deleted
                     if data:
                         fout.write(data)
@@ -226,25 +251,12 @@ class Assembler(Thread):
             nzf.md5sum = nzf.md5.digest()
 
 
-def file_has_articles(nzf):
-    """Do a quick check to see if any articles are present for this file.
-    Destructive: only to be used to differentiate between unknown encoding and no articles.
-    """
-    has = False
-    for article in nzf.decodetable:
-        sleep(0.01)
-        data = ArticleCache.do.load_article(article)
-        if data:
-            has = True
-    return has
-
-
 RE_SUBS = re.compile(r"\W+sub|subs|subpack|subtitle|subtitles(?![a-z])", re.I)
 SAFE_EXTS = (".mkv", ".mp4", ".avi", ".wmv", ".mpg", ".webm")
 
 
-def is_cloaked(nzo, path, names):
-    """ Return True if this is likely to be a cloaked encrypted post """
+def is_cloaked(nzo: NzbObject, path: str, names: List[str]) -> bool:
+    """Return True if this is likely to be a cloaked encrypted post"""
     fname = os.path.splitext(get_filename(path.lower()))[0]
     for name in names:
         name = get_filename(name.lower())
@@ -272,8 +284,8 @@ def is_cloaked(nzo, path, names):
     return False
 
 
-def check_encrypted_and_unwanted_files(nzo, filepath):
-    """ Combines check for unwanted and encrypted files to save on CPU and IO """
+def check_encrypted_and_unwanted_files(nzo: NzbObject, filepath: str) -> Tuple[bool, Optional[str]]:
+    """Combines check for unwanted and encrypted files to save on CPU and IO"""
     encrypted = False
     unwanted = None
 
@@ -289,7 +301,6 @@ def check_encrypted_and_unwanted_files(nzo, filepath):
             # Is it even a rarfile?
             if rarfile.is_rarfile(filepath):
                 # Open the rar
-                rarfile.UNRAR_TOOL = sabnzbd.newsunpack.RAR_COMMAND
                 zf = rarfile.RarFile(filepath, single_file_check=True)
 
                 # Check for encryption
@@ -319,28 +330,35 @@ def check_encrypted_and_unwanted_files(nzo, filepath):
                                     zf.setpassword(password)
                                 except rarfile.Error:
                                     # On weird passwords the setpassword() will fail
-                                    # but the actual rartest() will work
+                                    # but the actual testrar() will work
                                     pass
                                 try:
                                     zf.testrar()
                                     password_hit = password
                                     break
-                                except rarfile.RarCRCError:
-                                    # On CRC error we can continue!
-                                    password_hit = password
-                                    break
-                                except Exception as e:
-                                    # Did we start from the right volume?
-                                    if "need to start extraction from a previous volume" in str(e):
-                                        return encrypted, unwanted
-                                    # This one failed
+                                except rarfile.RarWrongPassword:
+                                    # This one really didn't work
                                     pass
+                                except rarfile.RarCRCError as e:
+                                    # CRC errors can be thrown for wrong password or
+                                    # missing the next volume (with correct password)
+                                    if match_str(str(e), ("cannot find volume", "unexpected end of archive")):
+                                        # We assume this one worked!
+                                        password_hit = password
+                                        break
+                                    # This one didn't work
+                                    pass
+                                except:
+                                    # All the other errors we skip, they might be fixable in post-proc.
+                                    # For example starting from the wrong volume, or damaged files
+                                    # This will cause the check to be performed again for the next rar, might
+                                    # be disk-intensive! Could be removed later and just accept the password.
+                                    return encrypted, unwanted
 
                         # Did any work?
                         if password_hit:
-                            # We always trust the user's input
-                            if not nzo.password:
-                                nzo.password = password_hit
+                            # Record the successful password
+                            nzo.correct_password = password_hit
                             # Don't check other files
                             logging.info('Password "%s" matches for job "%s"', password_hit, nzo.final_name)
                             nzo.encrypted = -1
@@ -354,7 +372,7 @@ def check_encrypted_and_unwanted_files(nzo, filepath):
                 if cfg.unwanted_extensions() and cfg.action_on_unwanted_extensions():
                     for somefile in zf.namelist():
                         logging.debug("File contains: %s", somefile)
-                        if get_ext(somefile).replace(".", "").lower() in cfg.unwanted_extensions():
+                        if has_unwanted_extension(somefile):
                             logging.debug("Unwanted file %s", somefile)
                             unwanted = somefile
                 zf.close()
@@ -366,9 +384,9 @@ def check_encrypted_and_unwanted_files(nzo, filepath):
     return encrypted, unwanted
 
 
-def nzo_filtered_by_rating(nzo):
-    if Rating.do and cfg.rating_enable() and cfg.rating_filter_enable() and (nzo.rating_filtered < 2):
-        rating = Rating.do.get_rating_by_nzo(nzo.nzo_id)
+def nzo_filtered_by_rating(nzo: NzbObject) -> Tuple[int, str]:
+    if cfg.rating_enable() and cfg.rating_filter_enable() and (nzo.rating_filtered < 2):
+        rating = sabnzbd.Rating.get_rating_by_nzo(nzo.nzo_id)
         if rating is not None:
             nzo.rating_filtered = 1
             reason = rating_filtered(rating, nzo.filename.lower(), True)
