@@ -1,5 +1,5 @@
 #!/usr/bin/python3 -OO
-# Copyright 2007-2020 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2022 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -18,7 +18,11 @@
 """
 sabnzbd.cfg - Configuration Parameters
 """
+
+import logging
 import re
+import argparse
+from typing import List, Tuple
 
 import sabnzbd
 from sabnzbd.config import (
@@ -28,13 +32,7 @@ from sabnzbd.config import (
     OptionDir,
     OptionStr,
     OptionList,
-    validate_octal,
-    validate_safedir,
-    all_lowercase,
     create_api_key,
-    validate_notempty,
-    clean_nice_ionice_parameters,
-    validate_strip_right_slash,
 )
 from sabnzbd.constants import (
     DEF_HOST,
@@ -48,9 +46,74 @@ from sabnzbd.constants import (
     DEF_FOLDER_MAX,
 )
 
+
 ##############################################################################
-# Email validation support
+# Default Validation handlers
 ##############################################################################
+class ErrorCatchingArgumentParser(argparse.ArgumentParser):
+    def error(self, status=0, message=None):
+        # Need to override so it doesn't raise SystemExit
+        if status:
+            raise ValueError
+
+
+def clean_nice_ionice_parameters(value):
+    """Verify that the passed parameters are not exploits"""
+    if value:
+        parser = ErrorCatchingArgumentParser()
+
+        # Nice parameters
+        parser.add_argument("-n", "--adjustment", type=int)
+
+        # Ionice parameters, not supporting -p
+        parser.add_argument("--classdata", type=int)
+        parser.add_argument("-c", "--class", type=int)
+        parser.add_argument("-t", "--ignore", action="store_true")
+
+        try:
+            parser.parse_args(value.split())
+        except ValueError:
+            # Also log at start-up if invalid parameter was set in the ini
+            msg = "%s: %s" % (T("Incorrect parameter"), value)
+            logging.error(msg)
+            return msg, None
+    return None, value
+
+
+def all_lowercase(value):
+    """Lowercase everything!"""
+    if isinstance(value, list):
+        # If list, for each item
+        return None, [item.lower() for item in value]
+    return None, value.lower()
+
+
+def validate_no_unc(root, value, default):
+    """Check if path isn't a UNC path"""
+    # Only need to check the 'value' part
+    if value and not value.startswith(r"\\"):
+        return validate_notempty(root, value, default)
+    else:
+        return T('UNC path "%s" not allowed here') % value, None
+
+
+def validate_single_tag(value: List[str]) -> Tuple[None, List[str]]:
+    """Don't split single indexer tags like "TV > HD"
+    into ['TV', '>', 'HD']
+    """
+    if len(value) == 3:
+        if value[1] == ">":
+            return None, [" ".join(value)]
+    return None, value
+
+
+def validate_strip_right_slash(value):
+    """Strips the right slash"""
+    if value:
+        return None, value.rstrip("/")
+    return None, value
+
+
 RE_VAL = re.compile(r"[^@ ]+@[^.@ ]+\.[^.@ ]")
 
 
@@ -68,7 +131,7 @@ def validate_email(value):
 
 
 def validate_server(value):
-    """ Check if server non-empty"""
+    """Check if server non-empty"""
     global email_endjob, email_full, email_rss
     if value == "" and (email_endjob() or email_full() or email_rss()):
         return T("Server address required"), None
@@ -76,20 +139,67 @@ def validate_server(value):
         return None, value
 
 
+def validate_script(value):
+    """Check if value is a valid script"""
+    if not sabnzbd.__INITIALIZED__ or (value and sabnzbd.filesystem.is_valid_script(value)):
+        return None, value
+    elif (value and value == "None") or not value:
+        return None, "None"
+    return T("%s is not a valid script") % value, None
+
+
+def validate_permissions(value: str):
+    """Check the permissions for correct input"""
+    # Octal verification
+    if not value:
+        return None, value
+    try:
+        oct_value = int(value, 8)
+        # Block setting it to 0
+        if not oct_value:
+            raise ValueError
+    except ValueError:
+        return T("%s is not a correct octal value") % value, None
+
+    # Check if we at least have user-permissions
+    if oct_value < int("700", 8):
+        sabnzbd.misc.helpful_warning(
+            T("Permissions setting of %s might deny SABnzbd access to the files and folders it creates."), value
+        )
+    return None, value
+
+
+def validate_safedir(root, value, default):
+    """Allow only when queues are empty and no UNC"""
+    if not sabnzbd.__INITIALIZED__ or (sabnzbd.PostProcessor.empty() and sabnzbd.NzbQueue.is_empty()):
+        return validate_no_unc(root, value, default)
+    else:
+        return T("Error: Queue not empty, cannot change folder."), None
+
+
+def validate_notempty(root, value, default):
+    """If value is empty, return default"""
+    if value:
+        return None, value
+    else:
+        return None, default
+
+
 ##############################################################################
 # Special settings
 ##############################################################################
-pre_script = OptionStr("misc", "pre_script", "None")
+pre_script = OptionStr("misc", "pre_script", "None", validation=validate_script)
 queue_complete = OptionStr("misc", "queue_complete")
 queue_complete_pers = OptionBool("misc", "queue_complete_pers", False)
-bandwidth_perc = OptionNumber("misc", "bandwidth_perc", 0, 0, 100)
+bandwidth_perc = OptionNumber("misc", "bandwidth_perc", 100, 0, 100)
 refresh_rate = OptionNumber("misc", "refresh_rate", 0)
+interface_settings = OptionStr("misc", "interface_settings")
 log_level = OptionNumber("logging", "log_level", 1, -1, 2)
-log_size = OptionStr("logging", "max_log_size", "5242880")
+log_size = OptionNumber("logging", "max_log_size", 5242880)
 log_backups = OptionNumber("logging", "log_backups", 5, 1, 1024)
 queue_limit = OptionNumber("misc", "queue_limit", 20, 0)
 
-configlock = OptionBool("misc", "config_lock", 0)
+configlock = OptionBool("misc", "config_lock", False)
 
 
 ##############################################################################
@@ -115,14 +225,13 @@ password = OptionPassword("misc", "password")
 bandwidth_max = OptionStr("misc", "bandwidth_max")
 cache_limit = OptionStr("misc", "cache_limit")
 web_dir = OptionStr("misc", "web_dir", DEF_STDINTF)
-web_color = OptionStr("misc", "web_color", "")
+web_color = OptionStr("misc", "web_color")
 https_cert = OptionDir("misc", "https_cert", "server.cert", create=False)
 https_key = OptionDir("misc", "https_key", "server.key", create=False)
 https_chain = OptionDir("misc", "https_chain", create=False)
 enable_https = OptionBool("misc", "enable_https", False)
 # 0=local-only, 1=nzb, 2=api, 3=full_api, 4=webui, 5=webui with login for external
 inet_exposure = OptionNumber("misc", "inet_exposure", 0, protect=True)
-local_ranges = OptionList("misc", "local_ranges", protect=True)
 api_key = OptionStr("misc", "api_key", create_api_key())
 nzb_key = OptionStr("misc", "nzb_key", create_api_key())
 
@@ -130,12 +239,16 @@ nzb_key = OptionStr("misc", "nzb_key", create_api_key())
 ##############################################################################
 # Config - Folders
 ##############################################################################
-umask = OptionStr("misc", "permissions", "", validation=validate_octal)
-download_dir = OptionDir("misc", "download_dir", DEF_DOWNLOAD_DIR, create=False, validation=validate_safedir)
+permissions = OptionStr("misc", "permissions", validation=validate_permissions)
+download_dir = OptionDir(
+    "misc", "download_dir", DEF_DOWNLOAD_DIR, create=False, apply_permissions=True, validation=validate_safedir
+)
 download_free = OptionStr("misc", "download_free")
 complete_dir = OptionDir(
-    "misc", "complete_dir", DEF_COMPLETE_DIR, create=False, apply_umask=True, validation=validate_notempty
+    "misc", "complete_dir", DEF_COMPLETE_DIR, create=False, apply_permissions=True, validation=validate_notempty
 )
+complete_free = OptionStr("misc", "complete_free")
+fulldisk_autoresume = OptionBool("misc", "fulldisk_autoresume", False)
 script_dir = OptionDir("misc", "script_dir", create=True, writable=False)
 nzb_backup_dir = OptionDir("misc", "nzb_backup_dir", DEF_NZBBACK_DIR)
 admin_dir = OptionDir("misc", "admin_dir", DEF_ADMIN_DIR, validation=validate_safedir)
@@ -154,14 +267,13 @@ top_only = OptionBool("misc", "top_only", False)
 sfv_check = OptionBool("misc", "sfv_check", True)
 quick_check_ext_ignore = OptionList("misc", "quick_check_ext_ignore", ["nfo", "sfv", "srr"])
 script_can_fail = OptionBool("misc", "script_can_fail", False)
-ssl_ciphers = OptionStr("misc", "ssl_ciphers", "")  # Now per-server setting
 enable_recursive = OptionBool("misc", "enable_recursive", True)
 flat_unpack = OptionBool("misc", "flat_unpack", False)
-par_option = OptionStr("misc", "par_option", "")
+par_option = OptionStr("misc", "par_option")
 pre_check = OptionBool("misc", "pre_check", False)
-nice = OptionStr("misc", "nice", "", validation=clean_nice_ionice_parameters)
+nice = OptionStr("misc", "nice", validation=clean_nice_ionice_parameters)
 win_process_prio = OptionNumber("misc", "win_process_prio", 3)
-ionice = OptionStr("misc", "ionice", "", validation=clean_nice_ionice_parameters)
+ionice = OptionStr("misc", "ionice", validation=clean_nice_ionice_parameters)
 fail_hopeless_jobs = OptionBool("misc", "fail_hopeless_jobs", True)
 fast_fail = OptionBool("misc", "fast_fail", True)
 autodisconnect = OptionBool("misc", "auto_disconnect", True)
@@ -180,14 +292,17 @@ replace_spaces = OptionBool("misc", "replace_spaces", False)
 replace_dots = OptionBool("misc", "replace_dots", False)
 safe_postproc = OptionBool("misc", "safe_postproc", True)
 pause_on_post_processing = OptionBool("misc", "pause_on_post_processing", False)
+enable_all_par = OptionBool("misc", "enable_all_par", False)
 sanitize_safe = OptionBool("misc", "sanitize_safe", False)
 cleanup_list = OptionList("misc", "cleanup_list")
 unwanted_extensions = OptionList("misc", "unwanted_extensions")
 action_on_unwanted_extensions = OptionNumber("misc", "action_on_unwanted_extensions", 0)
+unwanted_extensions_mode = OptionNumber("misc", "unwanted_extensions_mode", 0)
 new_nzb_on_failure = OptionBool("misc", "new_nzb_on_failure", False)
 history_retention = OptionStr("misc", "history_retention", "0")
 enable_meta = OptionBool("misc", "enable_meta", True)
 
+
 quota_size = OptionStr("misc", "quota_size")
 quota_day = OptionStr("misc", "quota_day")
 quota_resume = OptionBool("misc", "quota_resume", False)
@@ -220,13 +335,12 @@ rating_filter_pause_keywords = OptionStr("misc", "rating_filter_pause_keywords")
 ##############################################################################
 enable_tv_sorting = OptionBool("misc", "enable_tv_sorting", False)
 tv_sort_string = OptionStr("misc", "tv_sort_string")
-tv_sort_countries = OptionNumber("misc", "tv_sort_countries", 1)
-tv_categories = OptionList("misc", "tv_categories", "")
+tv_categories = OptionList("misc", "tv_categories", ["tv"])
 
 enable_movie_sorting = OptionBool("misc", "enable_movie_sorting", False)
 movie_sort_string = OptionStr("misc", "movie_sort_string")
 movie_sort_extra = OptionStr("misc", "movie_sort_extra", "-cd%1", strip=False)
-movie_extra_folders = OptionBool("misc", "movie_extra_folder", False)
+movie_extra_folder = OptionBool("misc", "movie_extra_folder", False)
 movie_categories = OptionList("misc", "movie_categories", ["movies"])
 
 enable_date_sorting = OptionBool("misc", "enable_date_sorting", False)
@@ -248,8 +362,9 @@ rss_rate = OptionNumber("misc", "rss_rate", 60, 15, 24 * 60)
 ampm = OptionBool("misc", "ampm", False)
 replace_illegal = OptionBool("misc", "replace_illegal", True)
 start_paused = OptionBool("misc", "start_paused", False)
-enable_all_par = OptionBool("misc", "enable_all_par", False)
+preserve_paused_state = OptionBool("misc", "preserve_paused_state", False)
 enable_par_cleanup = OptionBool("misc", "enable_par_cleanup", True)
+process_unpacked_par2 = OptionBool("misc", "process_unpacked_par2", True)
 enable_unrar = OptionBool("misc", "enable_unrar", True)
 enable_unzip = OptionBool("misc", "enable_unzip", True)
 enable_7zip = OptionBool("misc", "enable_7zip", True)
@@ -266,19 +381,18 @@ html_login = OptionBool("misc", "html_login", True)
 osx_menu = OptionBool("misc", "osx_menu", True)
 osx_speed = OptionBool("misc", "osx_speed", True)
 warn_dupl_jobs = OptionBool("misc", "warn_dupl_jobs", True)
-helpfull_warnings = OptionBool("misc", "helpfull_warnings", True)
+helpful_warnings = OptionBool("misc", "helpful_warnings", True)
 keep_awake = OptionBool("misc", "keep_awake", True)
 win_menu = OptionBool("misc", "win_menu", True)
 allow_incomplete_nzb = OptionBool("misc", "allow_incomplete_nzb", False)
-enable_bonjour = OptionBool("misc", "enable_bonjour", True)
-max_art_opt = OptionBool("misc", "max_art_opt", False)
+enable_broadcast = OptionBool("misc", "enable_broadcast", True)
 ipv6_hosting = OptionBool("misc", "ipv6_hosting", False)
 fixed_ports = OptionBool("misc", "fixed_ports", False)
 api_warnings = OptionBool("misc", "api_warnings", True, protect=True)
 disable_key = OptionBool("misc", "disable_api_key", False, protect=True)
 no_penalties = OptionBool("misc", "no_penalties", False)
 x_frame_options = OptionBool("misc", "x_frame_options", True)
-require_modern_tls = OptionBool("misc", "require_modern_tls", False)
+allow_old_ssl_tls = OptionBool("misc", "allow_old_ssl_tls", False)
 num_decoders = OptionNumber("misc", "num_decoders", 3)
 
 # Text values
@@ -286,16 +400,22 @@ rss_odd_titles = OptionList("misc", "rss_odd_titles", ["nzbindex.nl/", "nzbindex
 req_completion_rate = OptionNumber("misc", "req_completion_rate", 100.2, 100, 200)
 selftest_host = OptionStr("misc", "selftest_host", "self-test.sabnzbd.org")
 movie_rename_limit = OptionStr("misc", "movie_rename_limit", "100M")
+episode_rename_limit = OptionStr("misc", "episode_rename_limit", "20M")
 size_limit = OptionStr("misc", "size_limit", "0")
 show_sysload = OptionNumber("misc", "show_sysload", 2, 0, 2)
 history_limit = OptionNumber("misc", "history_limit", 10, 0)
 wait_ext_drive = OptionNumber("misc", "wait_ext_drive", 5, 1, 60)
 max_foldername_length = OptionNumber("misc", "max_foldername_length", DEF_FOLDER_MAX, 20, 65000)
-marker_file = OptionStr("misc", "nomedia_marker", "")
+marker_file = OptionStr("misc", "nomedia_marker")
 ipv6_servers = OptionNumber("misc", "ipv6_servers", 1, 0, 2)
 url_base = OptionStr("misc", "url_base", "/sabnzbd", validation=validate_strip_right_slash)
 host_whitelist = OptionList("misc", "host_whitelist", validation=all_lowercase)
+local_ranges = OptionList("misc", "local_ranges", protect=True)
 max_url_retries = OptionNumber("misc", "max_url_retries", 10, 1)
+downloader_sleep_time = OptionNumber("misc", "downloader_sleep_time", 10, 0)
+ssdp_broadcast_interval = OptionNumber("misc", "ssdp_broadcast_interval", 15, 1, 600)
+socks5_proxy_url = OptionStr("misc", "socks5_proxy_url")
+
 
 ##############################################################################
 # Config - Notifications
@@ -326,7 +446,7 @@ ncenter_prio_new_login = OptionBool("ncenter", "ncenter_prio_new_login", False)
 ncenter_prio_warning = OptionBool("ncenter", "ncenter_prio_warning", False)
 ncenter_prio_error = OptionBool("ncenter", "ncenter_prio_error", False)
 ncenter_prio_queue_done = OptionBool("ncenter", "ncenter_prio_queue_done", True)
-ncenter_prio_other = OptionBool("ncenter", "ncenter_prio_other", False)
+ncenter_prio_other = OptionBool("ncenter", "ncenter_prio_other", True)
 
 # [acenter]
 acenter_enable = OptionBool("acenter", "acenter_enable", sabnzbd.WIN32)
@@ -342,7 +462,7 @@ acenter_prio_new_login = OptionBool("acenter", "acenter_prio_new_login", False)
 acenter_prio_warning = OptionBool("acenter", "acenter_prio_warning", False)
 acenter_prio_error = OptionBool("acenter", "acenter_prio_error", False)
 acenter_prio_queue_done = OptionBool("acenter", "acenter_prio_queue_done", True)
-acenter_prio_other = OptionBool("acenter", "acenter_prio_other", False)
+acenter_prio_other = OptionBool("acenter", "acenter_prio_other", True)
 
 # [ntfosd]
 ntfosd_enable = OptionBool("ntfosd", "ntfosd_enable", not sabnzbd.WIN32 and not sabnzbd.DARWIN)
@@ -358,7 +478,7 @@ ntfosd_prio_new_login = OptionBool("ntfosd", "ntfosd_prio_new_login", False)
 ntfosd_prio_warning = OptionBool("ntfosd", "ntfosd_prio_warning", False)
 ntfosd_prio_error = OptionBool("ntfosd", "ntfosd_prio_error", False)
 ntfosd_prio_queue_done = OptionBool("ntfosd", "ntfosd_prio_queue_done", True)
-ntfosd_prio_other = OptionBool("ntfosd", "ntfosd_prio_other", False)
+ntfosd_prio_other = OptionBool("ntfosd", "ntfosd_prio_other", True)
 
 # [prowl]
 prowl_enable = OptionBool("prowl", "prowl_enable", False)
@@ -375,7 +495,7 @@ prowl_prio_new_login = OptionNumber("prowl", "prowl_prio_new_login", -3)
 prowl_prio_warning = OptionNumber("prowl", "prowl_prio_warning", -3)
 prowl_prio_error = OptionNumber("prowl", "prowl_prio_error", -3)
 prowl_prio_queue_done = OptionNumber("prowl", "prowl_prio_queue_done", 0)
-prowl_prio_other = OptionNumber("prowl", "prowl_prio_other", -3)
+prowl_prio_other = OptionNumber("prowl", "prowl_prio_other", 0)
 
 # [pushover]
 pushover_token = OptionStr("pushover", "pushover_token")
@@ -396,7 +516,7 @@ pushover_prio_new_login = OptionNumber("pushover", "pushover_prio_new_login", -3
 pushover_prio_warning = OptionNumber("pushover", "pushover_prio_warning", 1)
 pushover_prio_error = OptionNumber("pushover", "pushover_prio_error", 1)
 pushover_prio_queue_done = OptionNumber("pushover", "pushover_prio_queue_done", -1)
-pushover_prio_other = OptionNumber("pushover", "pushover_prio_other", -3)
+pushover_prio_other = OptionNumber("pushover", "pushover_prio_other", -1)
 
 # [pushbullet]
 pushbullet_enable = OptionBool("pushbullet", "pushbullet_enable")
@@ -414,12 +534,12 @@ pushbullet_prio_new_login = OptionBool("pushbullet", "pushbullet_prio_new_login"
 pushbullet_prio_warning = OptionBool("pushbullet", "pushbullet_prio_warning", False)
 pushbullet_prio_error = OptionBool("pushbullet", "pushbullet_prio_error", False)
 pushbullet_prio_queue_done = OptionBool("pushbullet", "pushbullet_prio_queue_done", False)
-pushbullet_prio_other = OptionBool("pushbullet", "pushbullet_prio_other", False)
+pushbullet_prio_other = OptionBool("pushbullet", "pushbullet_prio_other", True)
 
 # [nscript]
 nscript_enable = OptionBool("nscript", "nscript_enable")
 nscript_cats = OptionList("nscript", "nscript_cats", ["*"])
-nscript_script = OptionStr("nscript", "nscript_script")
+nscript_script = OptionStr("nscript", "nscript_script", validation=validate_script)
 nscript_parameters = OptionStr("nscript", "nscript_parameters")
 nscript_prio_startup = OptionBool("nscript", "nscript_prio_startup", True)
 nscript_prio_download = OptionBool("nscript", "nscript_prio_download", False)
@@ -432,7 +552,7 @@ nscript_prio_new_login = OptionBool("nscript", "nscript_prio_new_login", False)
 nscript_prio_warning = OptionBool("nscript", "nscript_prio_warning", False)
 nscript_prio_error = OptionBool("nscript", "nscript_prio_error", False)
 nscript_prio_queue_done = OptionBool("nscript", "nscript_prio_queue_done", True)
-nscript_prio_other = OptionBool("nscript", "nscript_prio_other", False)
+nscript_prio_other = OptionBool("nscript", "nscript_prio_other", True)
 
 
 ##############################################################################
@@ -454,3 +574,51 @@ def set_root_folders2():
     https_cert.set_root(admin_dir.get_path())
     https_key.set_root(admin_dir.get_path())
     https_chain.set_root(admin_dir.get_path())
+
+
+##############################################################################
+# Callbacks for settings
+##############################################################################
+def new_limit():
+    """Callback for article cache changes"""
+    sabnzbd.ArticleCache.new_limit(cache_limit.get_int())
+
+
+def guard_restart():
+    """Callback for config options requiring a restart"""
+    sabnzbd.RESTART_REQ = True
+
+
+def guard_top_only():
+    """Callback for change of top_only option"""
+    sabnzbd.NzbQueue.set_top_only(top_only())
+
+
+def guard_pause_on_pp():
+    """Callback for change of pause-download-on-pp"""
+    if pause_on_post_processing():
+        pass  # Not safe to idle downloader, because we don't know
+        # if post-processing is active now
+    else:
+        sabnzbd.Downloader.resume_from_postproc()
+
+
+def guard_quota_size():
+    """Callback for change of quota_size"""
+    sabnzbd.BPSMeter.change_quota()
+
+
+def guard_quota_dp():
+    """Callback for change of quota_day or quota_period"""
+    sabnzbd.Scheduler.restart()
+
+
+def guard_language():
+    """Callback for change of the interface language"""
+    sabnzbd.lang.set_language(language())
+    sabnzbd.api.clear_trans_cache()
+
+
+def guard_https_ver():
+    """Callback for change of https verification"""
+    sabnzbd.misc.set_https_verification(enable_https_verification())
