@@ -1,5 +1,5 @@
 #!/usr/bin/python3 -OO
-# Copyright 2007-2021 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2022 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -18,7 +18,7 @@
 """
 sabnzbd.config - Configuration Support
 """
-import argparse
+
 import logging
 import os
 import re
@@ -30,7 +30,7 @@ from urllib.parse import urlparse
 
 import configobj
 
-import sabnzbd.misc
+import sabnzbd
 from sabnzbd.constants import CONFIG_VERSION, NORMAL_PRIORITY, DEFAULT_PRIORITY
 from sabnzbd.decorators import synchronized
 from sabnzbd.filesystem import clip_path, real_path, create_real_path, renamer, remove_file, is_writable
@@ -52,7 +52,7 @@ RE_PARAMFINDER = re.compile(r"""(?:'.*?')|(?:".*?")|(?:[^'",\s][^,]*)""")
 
 
 class Option:
-    """ Basic option class, basic fields """
+    """Basic option class, basic fields"""
 
     def __init__(self, section: str, keyword: str, default_val: Any = None, add: bool = True, protect: bool = False):
         """Basic option
@@ -81,7 +81,7 @@ class Option:
             anchor[keyword] = self
 
     def get(self) -> Any:
-        """ Retrieve value field """
+        """Retrieve value field"""
         if self.__value is not None:
             return self.__value
         else:
@@ -91,11 +91,11 @@ class Option:
         return str(self.get())
 
     def get_dict(self, safe: bool = False) -> Dict[str, Any]:
-        """ Return value a dictionary """
+        """Return value a dictionary"""
         return {self.__keyword: self.get()}
 
     def set_dict(self, values: Dict[str, Any]):
-        """ Set value based on dictionary """
+        """Set value based on dictionary"""
         if not self.__protect:
             try:
                 self.set(values["value"])
@@ -103,7 +103,7 @@ class Option:
                 pass
 
     def set(self, value: Any):
-        """ Set new value, no validation """
+        """Set new value, no validation"""
         global modified
         if value is not None:
             if isinstance(value, list) or isinstance(value, dict) or value != self.__value:
@@ -116,11 +116,11 @@ class Option:
         return self.__default_val
 
     def callback(self, callback: Callable):
-        """ Set callback function """
+        """Set callback function"""
         self.__callback = callback
 
     def ident(self):
-        """ Return section-list and keyword """
+        """Return section-list and keyword"""
         return self.__sections, self.__keyword
 
 
@@ -145,7 +145,7 @@ class OptionNumber(Option):
         super().__init__(section, keyword, default_val, add=add, protect=protect)
 
     def set(self, value: Any):
-        """ set new value, limited by range """
+        """set new value, limited by range"""
         if value is not None:
             try:
                 if self.__int:
@@ -165,12 +165,12 @@ class OptionNumber(Option):
                 super().set(value)
 
     def __call__(self) -> Union[int, float]:
-        """ get() replacement """
+        """get() replacement"""
         return self.get()
 
 
 class OptionBool(Option):
-    """ Boolean option class, always returns 0 or 1."""
+    """Boolean option class, always returns 0 or 1."""
 
     def __init__(self, section: str, keyword: str, default_val: bool = False, add: bool = True, protect: bool = False):
         super().__init__(section, keyword, int(default_val), add=add, protect=protect)
@@ -180,19 +180,19 @@ class OptionBool(Option):
         super().set(sabnzbd.misc.int_conv(value))
 
     def __call__(self) -> int:
-        """ get() replacement """
+        """get() replacement"""
         return int(self.get())
 
 
 class OptionDir(Option):
-    """ Directory option class """
+    """Directory option class"""
 
     def __init__(
         self,
         section: str,
         keyword: str,
         default_val: str = "",
-        apply_umask: bool = False,
+        apply_permissions: bool = False,
         create: bool = True,
         validation: Optional[Callable] = None,
         writable: bool = True,
@@ -200,13 +200,13 @@ class OptionDir(Option):
     ):
         self.__validation: Optional[Callable] = validation
         self.__root: str = ""  # Base directory for relative paths
-        self.__apply_umask: bool = apply_umask
+        self.__apply_permissions: bool = apply_permissions
         self.__create: bool = create
         self.__writable: bool = writable
         super().__init__(section, keyword, default_val, add=add)
 
     def get(self) -> str:
-        """ Return value, corrected for platform """
+        """Return value, corrected for platform"""
         p = super().get()
         if sabnzbd.WIN32:
             return p.replace("/", "\\") if "/" in p else p
@@ -214,21 +214,23 @@ class OptionDir(Option):
             return p.replace("\\", "/") if "\\" in p else p
 
     def get_path(self) -> str:
-        """ Return full absolute path """
+        """Return full absolute path"""
         value = self.get()
         path = ""
         if value:
             path = real_path(self.__root, value)
             if self.__create and not os.path.exists(path):
-                _, path, _ = create_real_path(self.ident()[1], self.__root, value, self.__apply_umask, self.__writable)
+                _, path, _ = create_real_path(
+                    self.ident()[1], self.__root, value, self.__apply_permissions, self.__writable
+                )
         return path
 
     def get_clipped_path(self) -> str:
-        """ Return clipped full absolute path """
+        """Return clipped full absolute path"""
         return clip_path(self.get_path())
 
     def test_path(self) -> bool:
-        """ Return True if path exists """
+        """Return True if path exists"""
         value = self.get()
         if value:
             return os.path.exists(real_path(self.__root, value))
@@ -236,7 +238,7 @@ class OptionDir(Option):
             return False
 
     def set_root(self, root: str):
-        """ Set new root, is assumed to be valid """
+        """Set new root, is assumed to be valid"""
         self.__root = root
 
     def set(self, value: str, create: bool = False) -> Optional[str]:
@@ -253,23 +255,23 @@ class OptionDir(Option):
             if not error:
                 if value and (self.__create or create):
                     res, path, error = create_real_path(
-                        self.ident()[1], self.__root, value, self.__apply_umask, self.__writable
+                        self.ident()[1], self.__root, value, self.__apply_permissions, self.__writable
                     )
             if not error:
                 super().set(value)
         return error
 
     def set_create(self, value: bool):
-        """ Set auto-creation value """
+        """Set auto-creation value"""
         self.__create = value
 
     def __call__(self) -> str:
-        """ get() replacement """
+        """get() replacement"""
         return self.get()
 
 
 class OptionList(Option):
-    """ List option class """
+    """List option class"""
 
     def __init__(
         self,
@@ -286,7 +288,7 @@ class OptionList(Option):
         super().__init__(section, keyword, default_val, add=add, protect=protect)
 
     def set(self, value: Union[str, List]) -> Optional[str]:
-        """ Set the list given a comma-separated string or a list """
+        """Set the list given a comma-separated string or a list"""
         error = None
         if value is not None:
             if not isinstance(value, list):
@@ -301,20 +303,20 @@ class OptionList(Option):
         return error
 
     def get_string(self) -> str:
-        """ Return the list as a comma-separated string """
+        """Return the list as a comma-separated string"""
         return ", ".join(self.get())
 
     def default_string(self) -> str:
-        """ Return the default list as a comma-separated string """
+        """Return the default list as a comma-separated string"""
         return ", ".join(self.default())
 
     def __call__(self) -> List[str]:
-        """ get() replacement """
+        """get() replacement"""
         return self.get()
 
 
 class OptionStr(Option):
-    """ String class."""
+    """String class."""
 
     def __init__(
         self,
@@ -331,15 +333,15 @@ class OptionStr(Option):
         super().__init__(section, keyword, default_val, add=add, protect=protect)
 
     def get_float(self) -> float:
-        """ Return value converted to a float, allowing KMGT notation """
+        """Return value converted to a float, allowing KMGT notation"""
         return sabnzbd.misc.from_units(self.get())
 
     def get_int(self) -> int:
-        """ Return value converted to an int, allowing KMGT notation """
+        """Return value converted to an int, allowing KMGT notation"""
         return int(self.get_float())
 
     def set(self, value: Any) -> Optional[str]:
-        """ Set stripped value """
+        """Set stripped value"""
         error = None
         if isinstance(value, str) and self.__strip:
             value = value.strip()
@@ -351,46 +353,46 @@ class OptionStr(Option):
         return error
 
     def __call__(self) -> str:
-        """ get() replacement """
+        """get() replacement"""
         return self.get()
 
 
 class OptionPassword(Option):
-    """ Password class. """
+    """Password class."""
 
     def __init__(self, section: str, keyword: str, default_val: str = "", add: bool = True):
         self.get_string = self.get_stars
         super().__init__(section, keyword, default_val, add=add)
 
     def get(self) -> Optional[str]:
-        """ Return decoded password """
+        """Return decoded password"""
         return decode_password(super().get(), self.ident())
 
     def get_stars(self) -> Optional[str]:
-        """ Return non-descript asterisk string """
+        """Return non-descript asterisk string"""
         if self.get():
             return "*" * 10
         return ""
 
     def get_dict(self, safe: bool = False) -> Dict[str, str]:
-        """ Return value a dictionary """
+        """Return value a dictionary"""
         if safe:
             return {self.ident()[1]: self.get_stars()}
         else:
             return {self.ident()[1]: self.get()}
 
     def set(self, pw: str):
-        """ Set password, encode it """
+        """Set password, encode it"""
         if (pw is not None and pw == "") or (pw and pw.strip("*")):
             super().set(encode_password(pw))
 
     def __call__(self) -> str:
-        """ get() replacement """
+        """get() replacement"""
         return self.get()
 
 
 class ConfigServer:
-    """ Class defining a single server """
+    """Class defining a single server"""
 
     def __init__(self, name, values):
 
@@ -399,7 +401,7 @@ class ConfigServer:
 
         self.displayname = OptionStr(name, "displayname", add=False)
         self.host = OptionStr(name, "host", add=False)
-        self.port = OptionNumber(name, "port", 119, 0, 2 ** 16 - 1, add=False)
+        self.port = OptionNumber(name, "port", 119, 0, 2**16 - 1, add=False)
         self.timeout = OptionNumber(name, "timeout", 60, 20, 240, add=False)
         self.username = OptionStr(name, "username", add=False)
         self.password = OptionPassword(name, "password", add=False)
@@ -409,6 +411,7 @@ class ConfigServer:
         self.ssl_verify = OptionNumber(name, "ssl_verify", 2, add=False)
         self.ssl_ciphers = OptionStr(name, "ssl_ciphers", add=False)
         self.enable = OptionBool(name, "enable", True, add=False)
+        self.required = OptionBool(name, "required", False, add=False)
         self.optional = OptionBool(name, "optional", False, add=False)
         self.retention = OptionNumber(name, "retention", 0, add=False)
         self.expire_date = OptionStr(name, "expire_date", add=False)
@@ -422,7 +425,7 @@ class ConfigServer:
         add_to_database("servers", self.__name, self)
 
     def set_dict(self, values: Dict[str, Any]):
-        """ Set one or more fields, passed as dictionary """
+        """Set one or more fields, passed as dictionary"""
         # Replace usage_at_start value with most recent statistics if the user changes the quota value
         # Only when we are updating it from the Config
         if sabnzbd.WEBUI_READY and values.get("quota", "") != self.quota():
@@ -442,6 +445,7 @@ class ConfigServer:
             "ssl_ciphers",
             "send_group",
             "enable",
+            "required",
             "optional",
             "retention",
             "expire_date",
@@ -459,7 +463,7 @@ class ConfigServer:
             self.displayname.set(self.__name)
 
     def get_dict(self, safe: bool = False) -> Dict[str, Any]:
-        """ Return a dictionary with all attributes """
+        """Return a dictionary with all attributes"""
         output_dict = {}
         output_dict["name"] = self.__name
         output_dict["displayname"] = self.displayname()
@@ -476,6 +480,7 @@ class ConfigServer:
         output_dict["ssl_verify"] = self.ssl_verify()
         output_dict["ssl_ciphers"] = self.ssl_ciphers()
         output_dict["enable"] = self.enable()
+        output_dict["required"] = self.required()
         output_dict["optional"] = self.optional()
         output_dict["retention"] = self.retention()
         output_dict["expire_date"] = self.expire_date()
@@ -487,11 +492,11 @@ class ConfigServer:
         return output_dict
 
     def delete(self):
-        """ Remove from database """
+        """Remove from database"""
         delete_from_database("servers", self.__name)
 
     def rename(self, name: str):
-        """ Give server new display name """
+        """Give server new display name"""
         self.displayname.set(name)
 
     def ident(self) -> Tuple[str, str]:
@@ -499,7 +504,7 @@ class ConfigServer:
 
 
 class ConfigCat:
-    """ Class defining a single category """
+    """Class defining a single category"""
 
     def __init__(self, name: str, values: Dict[str, Any]):
         self.__name = name
@@ -509,14 +514,14 @@ class ConfigCat:
         self.pp = OptionStr(name, "pp", add=False)
         self.script = OptionStr(name, "script", "Default", add=False)
         self.dir = OptionDir(name, "dir", add=False, create=False)
-        self.newzbin = OptionList(name, "newzbin", add=False, validation=validate_single_tag)
+        self.newzbin = OptionList(name, "newzbin", add=False, validation=sabnzbd.cfg.validate_single_tag)
         self.priority = OptionNumber(name, "priority", DEFAULT_PRIORITY, add=False)
 
         self.set_dict(values)
         add_to_database("categories", self.__name, self)
 
     def set_dict(self, values: Dict[str, Any]):
-        """ Set one or more fields, passed as dictionary """
+        """Set one or more fields, passed as dictionary"""
         for kw in ("order", "pp", "script", "dir", "newzbin", "priority"):
             try:
                 value = values[kw]
@@ -525,7 +530,7 @@ class ConfigCat:
                 continue
 
     def get_dict(self, safe: bool = False) -> Dict[str, Any]:
-        """ Return a dictionary with all attributes """
+        """Return a dictionary with all attributes"""
         output_dict = {}
         output_dict["name"] = self.__name
         output_dict["order"] = self.order()
@@ -537,19 +542,19 @@ class ConfigCat:
         return output_dict
 
     def delete(self):
-        """ Remove from database """
+        """Remove from database"""
         delete_from_database("categories", self.__name)
 
 
 class OptionFilters(Option):
-    """ Filter list class """
+    """Filter list class"""
 
     def __init__(self, section, keyword, add=True):
         super().__init__(section, keyword, add=add)
         self.set([])
 
     def move(self, current: int, new: int):
-        """ Move filter from position 'current' to 'new' """
+        """Move filter from position 'current' to 'new'"""
         lst = self.get()
         try:
             item = lst.pop(current)
@@ -570,7 +575,7 @@ class OptionFilters(Option):
         self.set(lst)
 
     def delete(self, pos: int):
-        """ Remove filter 'pos' """
+        """Remove filter 'pos'"""
         lst = self.get()
         try:
             lst.pop(pos)
@@ -579,14 +584,14 @@ class OptionFilters(Option):
         self.set(lst)
 
     def get_dict(self, safe: bool = False) -> Dict[str, str]:
-        """ Return filter list as a dictionary with keys 'filter[0-9]+' """
+        """Return filter list as a dictionary with keys 'filter[0-9]+'"""
         output_dict = {}
         for n, rss_filter in enumerate(self.get()):
             output_dict[f"filter{n}"] = rss_filter
         return output_dict
 
     def set_dict(self, values: Dict[str, Any]):
-        """ Create filter list from dictionary with keys 'filter[0-9]+' """
+        """Create filter list from dictionary with keys 'filter[0-9]+'"""
         filters = []
         # We don't know how many filters there are, so just assume all values are filters
         for n in range(len(values)):
@@ -597,12 +602,12 @@ class OptionFilters(Option):
             self.set(filters)
 
     def __call__(self) -> List[List[str]]:
-        """ get() replacement """
+        """get() replacement"""
         return self.get()
 
 
 class ConfigRSS:
-    """ Class defining a single Feed definition """
+    """Class defining a single Feed definition"""
 
     def __init__(self, name, values):
         self.__name = name
@@ -621,7 +626,7 @@ class ConfigRSS:
         add_to_database("rss", self.__name, self)
 
     def set_dict(self, values: Dict[str, Any]):
-        """ Set one or more fields, passed as dictionary """
+        """Set one or more fields, passed as dictionary"""
         for kw in ("uri", "cat", "pp", "script", "priority", "enable"):
             try:
                 value = values[kw]
@@ -631,7 +636,7 @@ class ConfigRSS:
         self.filters.set_dict(values)
 
     def get_dict(self, safe: bool = False) -> Dict[str, Any]:
-        """ Return a dictionary with all attributes """
+        """Return a dictionary with all attributes"""
         output_dict = {}
         output_dict["name"] = self.__name
         output_dict["uri"] = self.uri()
@@ -646,11 +651,11 @@ class ConfigRSS:
         return output_dict
 
     def delete(self):
-        """ Remove from database """
+        """Remove from database"""
         delete_from_database("rss", self.__name)
 
     def rename(self, new_name: str):
-        """ Update the name and the saved entries """
+        """Update the name and the saved entries"""
         delete_from_database("rss", self.__name)
         sabnzbd.RSSReader.rename(self.__name, new_name)
         self.__name = new_name
@@ -662,7 +667,7 @@ class ConfigRSS:
 
 @synchronized(CONFIG_LOCK)
 def add_to_database(section, keyword, obj):
-    """ add object as section/keyword to INI database """
+    """add object as section/keyword to INI database"""
     global database
     if section not in database:
         database[section] = {}
@@ -671,7 +676,7 @@ def add_to_database(section, keyword, obj):
 
 @synchronized(CONFIG_LOCK)
 def delete_from_database(section, keyword):
-    """ Remove section/keyword from INI database """
+    """Remove section/keyword from INI database"""
     global database, CFG, modified
     del database[section][keyword]
     if section == "servers" and "[" in keyword:
@@ -725,7 +730,7 @@ def get_dconfig(section, keyword, nested=False):
 
 
 def get_config(section, keyword):
-    """ Return a config object, based on 'section', 'keyword' """
+    """Return a config object, based on 'section', 'keyword'"""
     try:
         return database[section][keyword]
     except KeyError:
@@ -734,7 +739,7 @@ def get_config(section, keyword):
 
 
 def set_config(kwargs):
-    """ Set a config item, using values in dictionary """
+    """Set a config item, using values in dictionary"""
     try:
         item = database[kwargs.get("section")][kwargs.get("keyword")]
     except KeyError:
@@ -744,7 +749,7 @@ def set_config(kwargs):
 
 
 def delete(section: str, keyword: str):
-    """ Delete specific config item """
+    """Delete specific config item"""
     try:
         database[section][keyword].delete()
     except KeyError:
@@ -842,7 +847,7 @@ def _read_config(path, try_backup=False):
 
 @synchronized(SAVE_CONFIG_LOCK)
 def save_config(force=False):
-    """ Update Setup file with current option values """
+    """Update Setup file with current option values"""
     global CFG, database, modified
 
     if not (modified or force):
@@ -1011,21 +1016,11 @@ def get_filename():
     return CFG.filename
 
 
-##############################################################################
-# Default Validation handlers
-##############################################################################
 __PW_PREFIX = "!!!encoded!!!"
 
 
-class ErrorCatchingArgumentParser(argparse.ArgumentParser):
-    def error(self, status=0, message=None):
-        # Need to override so it doesn't raise SystemExit
-        if status:
-            raise ValueError
-
-
 def encode_password(pw):
-    """ Encode password in hexadecimal if needed """
+    """Encode password in hexadecimal if needed"""
     enc = False
     if pw:
         encPW = __PW_PREFIX
@@ -1057,90 +1052,6 @@ def decode_password(pw, name):
         return pw
 
 
-def clean_nice_ionice_parameters(value):
-    """ Verify that the passed parameters are not exploits """
-    if value:
-        parser = ErrorCatchingArgumentParser()
-
-        # Nice parameters
-        parser.add_argument("-n", "--adjustment", type=int)
-
-        # Ionice parameters, not supporting -p
-        parser.add_argument("--classdata", type=int)
-        parser.add_argument("-c", "--class", type=int)
-        parser.add_argument("-t", "--ignore", action="store_true")
-
-        try:
-            parser.parse_args(value.split())
-        except ValueError:
-            # Also log at start-up if invalid parameter was set in the ini
-            msg = "%s: %s" % (T("Incorrect parameter"), value)
-            logging.error(msg)
-            return msg, None
-    return None, value
-
-
-def all_lowercase(value):
-    """ Lowercase everything! """
-    if isinstance(value, list):
-        # If list, for each item
-        return None, [item.lower() for item in value]
-    return None, value.lower()
-
-
-def validate_octal(value):
-    """ Check if string is valid octal number """
-    if not value:
-        return None, value
-    try:
-        int(value, 8)
-        return None, value
-    except:
-        return T("%s is not a correct octal value") % value, None
-
-
-def validate_no_unc(root, value, default):
-    """ Check if path isn't a UNC path """
-    # Only need to check the 'value' part
-    if value and not value.startswith(r"\\"):
-        return validate_notempty(root, value, default)
-    else:
-        return T('UNC path "%s" not allowed here') % value, None
-
-
-def validate_safedir(root, value, default):
-    """Allow only when queues are empty and no UNC"""
-    if not sabnzbd.__INITIALIZED__ or (sabnzbd.PostProcessor.empty() and sabnzbd.NzbQueue.is_empty()):
-        return validate_no_unc(root, value, default)
-    else:
-        return T("Error: Queue not empty, cannot change folder."), None
-
-
-def validate_notempty(root, value, default):
-    """ If value is empty, return default """
-    if value:
-        return None, value
-    else:
-        return None, default
-
-
-def validate_strip_right_slash(value):
-    """Strips the right slash"""
-    if value:
-        return None, value.rstrip("/")
-    return None, value
-
-
-def validate_single_tag(value):
-    """Don't split single indexer tags like "TV > HD"
-    into ['TV', '>', 'HD']
-    """
-    if len(value) == 3:
-        if value[1] == ">":
-            return None, [" ".join(value)]
-    return None, value
-
-
 def create_api_key():
-    """ Return a new randomized API_KEY """
+    """Return a new randomized API_KEY"""
     return uuid.uuid4().hex
