@@ -1,5 +1,5 @@
 #!/usr/bin/python3 -OO
-# Copyright 2007-2020 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2022 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -17,14 +17,13 @@
 
 import os
 from collections import namedtuple
-from re import compile
 
 CONFIG_VERSION = 19
 
 QUEUE_VERSION = 10
 POSTPROC_QUEUE_VERSION = 2
 
-REC_RAR_VERSION = 500
+REC_RAR_VERSION = 550
 
 PNFO = namedtuple(
     "PNFO",
@@ -40,10 +39,11 @@ ANFO = namedtuple("ANFO", "article_sum cache_size cache_limit")
 # Leave some space for "_UNPACK_" which we append during post-proc
 # Or, when extra ".1", ".2" etc. are added for identically named jobs
 DEF_FOLDER_MAX = 256 - 10
+DEF_FILE_MAX = 255 - 10  # max filename length on modern filesystems, minus some room for extra chars later on
 
-GIGI = float(2 ** 30)
-MEBI = float(2 ** 20)
-KIBI = float(2 ** 10)
+GIGI = float(2**30)
+MEBI = float(2**20)
+KIBI = float(2**10)
 
 BYTES_FILE_NAME_OLD = "totals9.sab"
 BYTES_FILE_NAME = "totals10.sab"
@@ -52,6 +52,7 @@ QUEUE_FILE_NAME = QUEUE_FILE_TMPL % QUEUE_VERSION
 POSTPROC_QUEUE_FILE_NAME = "postproc%s.sab" % POSTPROC_QUEUE_VERSION
 RSS_FILE_NAME = "rss_data.sab"
 SCAN_FILE_NAME = "watched_data2.sab"
+RATING_FILE_NAME = "Rating.sab"
 FUTURE_Q_FOLDER = "future"
 JOB_ADMIN = "__ADMIN__"
 VERIFIED_FILE = "__verified__"
@@ -73,7 +74,6 @@ DEF_INTERFACES = "interfaces"
 DEF_EMAIL_TMPL = "email"
 DEF_STDCONFIG = "Config"
 DEF_STDINTF = "Glitter"
-DEF_SKIN_COLORS = {"Glitter": "Default", "plush": "gold"}
 DEF_MAIN_TMPL = os.path.normpath("templates/main.tmpl")
 DEF_INI_FILE = "sabnzbd.ini"
 DEF_HOST = "127.0.0.1"
@@ -87,7 +87,6 @@ DEF_ARTICLE_CACHE_MAX = "1G"
 DEF_TIMEOUT = 60
 DEF_SCANRATE = 5
 MAX_WARNINGS = 20
-MAX_WIN_DFOLDER = 60
 MAX_BAD_ARTICLES = 5
 
 # Constants affecting download performance
@@ -121,27 +120,16 @@ VALID_NZB_FILES = (".nzb", ".gz", ".bz2")
 
 CHEETAH_DIRECTIVES = {"directiveStartToken": "<!--#", "directiveEndToken": "#-->", "prioritizeSearchListOverSelf": True}
 
-IGNORED_FOLDERS = ("@eaDir", ".appleDouble")
+IGNORED_FILES_AND_FOLDERS = ("@eaDir", ".appleDouble", ".DS_Store")
+IGNORED_MOVIE_FOLDERS = ("video_ts", "audio_ts", "bdmv")
 
-# (MATCHER, [EXTRA, MATCHERS])
-series_match = [
-    (compile(r"( [sS]|[\d]+)x(\d+)"), [compile(r"^[-\.]+([sS]|[\d])+x(\d+)"), compile(r"^[-\.](\d+)")]),  # 1x01
-    (
-        compile(r"[Ss](\d+)[\.\-]?[Ee](\d+)"),  # S01E01
-        [compile(r"^[-\.]+[Ss](\d+)[\.\-]?[Ee](\d+)"), compile(r"^[-\.](\d+)")],
-    ),
-    (compile(r"[ \-_\.](\d)(\d{2,2})[ \-_\.]"), []),  # .101. / _101_ / etc.
-    (compile(r"[ \-_\.](\d)(\d{2,2})$"), []),  # .101 at end of title
+EXCLUDED_GUESSIT_PROPERTIES = [
+    "part",
 ]
 
-date_match = [r"(\d{4})\W(\d{1,2})\W(\d{1,2})", r"(\d{1,2})\W(\d{1,2})\W(\d{4})"]  # 2008-10-16  # 10.16.2008
-
-year_match = r"[\W]([1|2]\d{3})([^\w]|$)"  # Something '(YYYY)' or '.YYYY.' or ' YYYY '
-
-sample_match = r"((^|[\W_])(sample|proof))"  # something-sample or something-proof
-
 
 class Status:
+    IDLE = "Idle"  # Q: Nothing in the queue
     COMPLETED = "Completed"  # PP: Job is finished
     CHECKING = "Checking"  # Q:  Pre-check is running
     DOWNLOADING = "Downloading"  # Q:  Normal downloading
