@@ -1,5 +1,5 @@
 #!/usr/bin/python3 -OO
-# Copyright 2007-2020 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2022 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -31,128 +31,38 @@ from sabnzbd.constants import RSS_FILE_NAME, DEFAULT_PRIORITY, DUP_PRIORITY
 from sabnzbd.decorators import synchronized
 import sabnzbd.config as config
 import sabnzbd.cfg as cfg
-from sabnzbd.misc import cat_convert, wildcard_to_re, cat_to_opts, match_str, from_units, int_conv, get_base_url
+from sabnzbd.misc import (
+    cat_convert,
+    convert_filter,
+    cat_to_opts,
+    match_str,
+    from_units,
+    int_conv,
+    get_base_url,
+    helpful_warning,
+)
 import sabnzbd.emailer as emailer
 
 import feedparser
 
-__RSS = None  # Global pointer to RSS-scanner instance
-
 
 ##############################################################################
 # Wrapper functions
 ##############################################################################
 
 
-def init():
-    global __RSS
-    __RSS = RSSQueue()
-
-
-def stop():
-    global __RSS
-    if __RSS:
-        __RSS.stop()
-        try:
-            __RSS.join()
-        except:
-            pass
-
-
-def run_feed(feed, download, ignoreFirst=False, force=False, readout=True):
-    global __RSS
-    if __RSS:
-        return __RSS.run_feed(feed, download, ignoreFirst, force=force, readout=readout)
-
-
-def show_result(feed):
-    global __RSS
-    if __RSS:
-        return __RSS.show_result(feed)
-
-
-def flag_downloaded(feed, fid):
-    global __RSS
-    if __RSS:
-        __RSS.flag_downloaded(feed, fid)
-
-
-def lookup_url(feed, fid):
-    global __RSS
-    if __RSS:
-        return __RSS.lookup_url(feed, fid)
-
-
-def run_method():
-    global __RSS
-    if __RSS:
-        return __RSS.run()
-    else:
-        return None
-
-
-def next_run(t=None):
-    global __RSS
-    if __RSS:
-        if t:
-            __RSS.next_run = t
-        else:
-            return __RSS.next_run
-    else:
-        return time.time()
-
-
-def save():
-    global __RSS
-    if __RSS:
-        __RSS.save()
-
-
-def clear_feed(feed):
-    global __RSS
-    if __RSS:
-        __RSS.clear_feed(feed)
-
-
-def clear_downloaded(feed):
-    global __RSS
-    if __RSS:
-        __RSS.clear_downloaded(feed)
-
-
-##############################################################################
-
-
 def notdefault(item):
-    """ Return True if not 'Default|''|*' """
+    """Return True if not 'Default|''|*'"""
     return bool(item) and str(item).lower() not in ("default", "*", "", str(DEFAULT_PRIORITY))
 
 
-def convert_filter(text):
-    """Return compiled regex.
-    If string starts with re: it's a real regex
-    else quote all regex specials, replace '*' by '.*'
-    """
-    text = text.strip().lower()
-    if text.startswith("re:"):
-        txt = text[3:].strip()
-    else:
-        txt = wildcard_to_re(text)
-    try:
-        return re.compile(txt, re.I)
-    except:
-        logging.debug("Could not compile regex: %s", text)
-        return None
-
-
 def remove_obsolete(jobs, new_jobs):
     """Expire G/B links that are not in new_jobs (mark them 'X')
     Expired links older than 3 days are removed from 'jobs'
     """
     now = time.time()
     limit = now - 259200  # 3days (3x24x3600)
-    olds = list(jobs.keys())
-    for old in olds:
+    for old in list(jobs):
         tm = jobs[old]["time"]
         if old not in new_jobs:
             if jobs[old].get("status", " ")[0] in ("G", "B"):
@@ -162,23 +72,23 @@ def remove_obsolete(jobs, new_jobs):
             del jobs[old]
 
 
-LOCK = threading.RLock()
+RSS_LOCK = threading.RLock()
 _RE_SP = re.compile(r"s*(\d+)[ex](\d+)", re.I)
 _RE_SIZE1 = re.compile(r"Size:\s*(\d+\.\d+\s*[KMG]{0,1})B\W*", re.I)
 _RE_SIZE2 = re.compile(r"\W*(\d+\.\d+\s*[KMG]{0,1})B\W*", re.I)
 
 
-class RSSQueue:
+class RSSReader:
     def __init__(self):
         self.jobs = {}
         self.next_run = time.time()
         self.shutdown = False
 
         try:
-            self.jobs = sabnzbd.load_admin(RSS_FILE_NAME)
+            self.jobs = sabnzbd.filesystem.load_admin(RSS_FILE_NAME)
             if self.jobs:
                 for feed in self.jobs:
-                    remove_obsolete(self.jobs[feed], list(self.jobs[feed].keys()))
+                    remove_obsolete(self.jobs[feed], list(self.jobs[feed]))
         except:
             logging.warning(T("Cannot read %s"), RSS_FILE_NAME)
             logging.info("Traceback: ", exc_info=True)
@@ -212,9 +122,9 @@ class RSSQueue:
     def stop(self):
         self.shutdown = True
 
-    @synchronized(LOCK)
+    @synchronized(RSS_LOCK)
     def run_feed(self, feed=None, download=False, ignoreFirst=False, force=False, readout=True):
-        """ Run the query for one URI and apply filters """
+        """Run the query for one URI and apply filters"""
         self.shutdown = False
 
         if not feed:
@@ -233,7 +143,6 @@ class RSSQueue:
 
         uris = feeds.uri()
         defCat = feeds.cat()
-        import sabnzbd.api
 
         if not notdefault(defCat) or defCat not in sabnzbd.api.list_cats(default=False):
             defCat = None
@@ -319,7 +228,7 @@ class RSSQueue:
 
                 if msg:
                     # We need to escape any "%20" that could be in the warning due to the URL's
-                    logging.warning_helpful(urllib.parse.unquote(msg))
+                    helpful_warning(urllib.parse.unquote(msg))
                 elif not entries:
                     msg = T("RSS Feed %s was empty") % uri
                     logging.info(msg)
@@ -552,20 +461,20 @@ class RSSQueue:
         return msg
 
     def run(self):
-        """ Run all the URI's and filters """
+        """Run all the URI's and filters"""
         if not sabnzbd.PAUSED_ALL:
             active = False
             if self.next_run < time.time():
-                self.next_run = time.time() + cfg.rss_rate.get() * 60
+                self.next_run = time.time() + cfg.rss_rate() * 60
             feeds = config.get_rss()
             try:
                 for feed in feeds:
-                    if feeds[feed].enable.get():
+                    if feeds[feed].enable():
                         logging.info('Starting scheduled RSS read-out for "%s"', feed)
                         active = True
                         self.run_feed(feed, download=True, ignoreFirst=True)
                         # Wait 15 seconds, else sites may get irritated
-                        for unused in range(15):
+                        for _ in range(15):
                             if self.shutdown:
                                 return
                             else:
@@ -579,7 +488,7 @@ class RSSQueue:
                 self.save()
                 logging.info("Finished scheduled RSS read-outs")
 
-    @synchronized(LOCK)
+    @synchronized(RSS_LOCK)
     def show_result(self, feed):
         if feed in self.jobs:
             try:
@@ -589,16 +498,22 @@ class RSSQueue:
         else:
             return {}
 
-    @synchronized(LOCK)
+    @synchronized(RSS_LOCK)
     def save(self):
-        sabnzbd.save_admin(self.jobs, RSS_FILE_NAME)
+        sabnzbd.filesystem.save_admin(self.jobs, RSS_FILE_NAME)
 
-    @synchronized(LOCK)
+    @synchronized(RSS_LOCK)
     def delete(self, feed):
         if feed in self.jobs:
             del self.jobs[feed]
 
-    @synchronized(LOCK)
+    @synchronized(RSS_LOCK)
+    def rename(self, old_feed, new_feed):
+        if old_feed in self.jobs:
+            old_data = self.jobs.pop(old_feed)
+            self.jobs[new_feed] = old_data
+
+    @synchronized(RSS_LOCK)
     def flag_downloaded(self, feed, fid):
         if feed in self.jobs:
             lst = self.jobs[feed]
@@ -607,7 +522,7 @@ class RSSQueue:
                     lst[link]["status"] = "D"
                     lst[link]["time_downloaded"] = time.localtime()
 
-    @synchronized(LOCK)
+    @synchronized(RSS_LOCK)
     def lookup_url(self, feed, url):
         if url and feed in self.jobs:
             lst = self.jobs[feed]
@@ -616,13 +531,13 @@ class RSSQueue:
                     return lst[link]
         return None
 
-    @synchronized(LOCK)
+    @synchronized(RSS_LOCK)
     def clear_feed(self, feed):
         # Remove any previous references to this feed name, and start fresh
         if feed in self.jobs:
             del self.jobs[feed]
 
-    @synchronized(LOCK)
+    @synchronized(RSS_LOCK)
     def clear_downloaded(self, feed):
         # Mark downloaded jobs, so that they won't be displayed any more.
         if feed in self.jobs:
@@ -707,7 +622,7 @@ def _HandleLink(
     priority=DEFAULT_PRIORITY,
     rule=0,
 ):
-    """ Process one link """
+    """Process one link"""
     if script == "":
         script = None
     if pp == "":
@@ -738,7 +653,7 @@ def _HandleLink(
         jobs[link]["status"] = "D"
         jobs[link]["time_downloaded"] = time.localtime()
         logging.info("Adding %s (%s) to queue", link, title)
-        sabnzbd.add_url(link, pp=pp, script=script, cat=cat, priority=priority, nzbname=nzbname)
+        sabnzbd.urlgrabber.add_url(link, pp=pp, script=script, cat=cat, priority=priority, nzbname=nzbname)
     else:
         if star:
             jobs[link]["status"] = flag + "*"
@@ -823,7 +738,7 @@ def _get_link(entry):
 
 
 def special_rss_site(url):
-    """ Return True if url describes an RSS site with odd titles """
+    """Return True if url describes an RSS site with odd titles"""
     return cfg.rss_filenames() or match_str(url, cfg.rss_odd_titles())
 
 
