@@ -1,6 +1,6 @@
 #!/usr/bin/python3 -OO
 # -*- coding: UTF-8 -*-
-# Copyright 2012-2020 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2022 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -113,8 +113,11 @@ SKIN_TEXT = {
     "day-of-month": TT("Day of month"),
     "thisWeek": TT("This week"),
     "thisMonth": TT("This month"),
+    "selectedDates": TT("Selected date range"),
     "today": TT("Today"),
     "total": TT("Total"),
+    "custom": TT("Custom"),
+    "speed": TT("Speed"),
     "on": TT("on"),
     "off": TT("off"),
     "parameters": TT("Parameters"),  #: Config: startup parameters of SABnzbd
@@ -345,6 +348,9 @@ SKIN_TEXT = {
     "opt-enable_https": TT("Enable HTTPS"),
     "opt-notInstalled": TT("not installed"),
     "explain-enable_https": TT("Enable accessing the interface from a HTTPS address."),
+    "explain-enable_https_warning": TT(
+        "Modern web browsers and other clients will not accept self-signed certificates and will give a warning and/or won't connect at all."
+    ),
     "opt-https_port": TT("HTTPS Port"),
     "explain-https_port": TT("If empty, the standard port will only listen to HTTPS."),
     "opt-https_cert": TT("HTTPS Certificate"),
@@ -391,12 +397,8 @@ SKIN_TEXT = {
     "explain-nzbkey": TT("This key will allow 3rd party programs to add NZBs to SABnzbd."),
     "button-apikey": TT("Generate New Key"),
     "explain-qr-code": TT("API Key QR Code"),  #: Explanation for QR code of APIKEY
-    "opt-local_ranges": TT("List of local network ranges"),
-    "explain-local_ranges": TT('All local network addresses start with these prefixes (often "192.168.1.")'),
     "opt-inet_exposure": TT("External internet access"),
-    "explain-inet_exposure": TT(
-        "You can set access rights for systems outside your local network. Requires List of local network ranges to be defined."
-    ),
+    "explain-inet_exposure": TT("You can set access rights for systems outside your local network."),
     "inet-local": TT("No access"),  # Selection value for external access
     "inet-nzb": TT("Add NZB files "),  # Selection value for external access
     "inet-api": TT("API (no Config)"),  # Selection value for external access
@@ -422,6 +424,12 @@ SKIN_TEXT = {
     "explain-complete_dir": TT(
         "Location to store finished, fully processed downloads.<br /><i>Can be overruled by user-defined categories.</i>"
     ),
+    "opt-complete_free": TT("Minimum Free Space for Completed Download Folder"),
+    "explain-complete_free": TT("Will not work if a category folder is on a different disk."),
+    "opt-fulldisk_autoresume": TT("Auto resume"),
+    "explain-fulldisk_autoresume": TT(
+        "Downloading will automatically resume if the minimum free space is available again.<br />Applies to both the Temporary and Complete Download Folder.<br />Checked every few minutes."
+    ),
     "opt-permissions": TT("Permissions for completed downloads"),
     "explain-permissions": TT(
         'Set permissions pattern for completed files/folders.<br /><i>In octal notation. For example: "755" or "777"</i>'
@@ -459,7 +467,9 @@ SKIN_TEXT = {
     "opt-top_only": TT("Only Get Articles for Top of Queue"),
     "explain-top_only": TT("Enable for less memory usage. Disable to prevent slow jobs from blocking the queue."),
     "opt-safe_postproc": TT("Post-Process Only Verified Jobs"),
-    "explain-safe_postproc": TT("Only perform post-processing on jobs that passed all PAR2 checks."),
+    "explain-safe_postproc": TT(
+        "Only unpack and run scripts on jobs that passed the verification stage. If turned off, all jobs will be marked as Completed even if they are incomplete."
+    ),
     "opt-pause_on_pwrar": TT("Action when encrypted RAR is downloaded"),
     "explain-pause_on_pwrar": TT('In case of "Pause", you\'ll need to set a password and resume the job.'),
     "opt-no_dupes": TT("Detect Duplicate Downloads"),
@@ -481,9 +491,13 @@ SKIN_TEXT = {
     "nodupes-tag": TT("Tag job"),  #: Four way switch for duplicates
     "abort": TT("Abort"),  #: Three way switch for encrypted posts
     "opt-action_on_unwanted_extensions": TT("Action when unwanted extension detected"),
-    "explain-action_on_unwanted_extensions": TT("Action when an unwanted extension is detected in RAR files"),
+    "explain-action_on_unwanted_extensions": TT("Action when an unwanted extension is detected"),
     "opt-unwanted_extensions": TT("Unwanted extensions"),
-    "explain-unwanted_extensions": TT("List all unwanted extensions. For example: <b>exe</b> or <b>exe, com</b>"),
+    "unwanted_extensions_blacklist": TT("Blacklist"),
+    "unwanted_extensions_whitelist": TT("Whitelist"),
+    "explain-unwanted_extensions": TT(
+        "Select a mode and list all (un)wanted extensions. For example: <b>exe</b> or <b>exe, com</b>"
+    ),
     "opt-sfv_check": TT("Enable SFV-based checks"),
     "explain-sfv_check": TT("Do an extra verification based on SFV files."),
     "opt-script_can_fail": TT("User script can flag job as failed"),
@@ -548,10 +562,15 @@ SKIN_TEXT = {
     "explain-deobfuscate_final_filenames": TT(
         "If filenames of (large) files in the final folder look obfuscated or meaningless they will be renamed to the job name."
     ),
+    "explain-deobfuscate_final_filenames-ext": TT(
+        "Additionally, attempts to set the correct file extension based on the file signature if the extension is not present or meaningless."
+    ),
     "opt-enable_https_verification": TT("HTTPS certificate verification"),
     "explain-enable_https_verification": TT(
         "Verify certificates when connecting to indexers and RSS-sources using HTTPS."
     ),
+    "opt-socks5_proxy_url": TT("SOCKS5 Proxy"),
+    "explain-socks5_proxy_url": TT("Use the specified SOCKS5 proxy for all outgoing connections."),
     "swtag-server": TT("Server"),
     "swtag-queue": TT("Queue"),
     "swtag-pp": TT("Post processing"),
@@ -610,6 +629,11 @@ SKIN_TEXT = {
     "srv-password": TT("Password"),  #: Server password
     "srv-timeout": TT("Timeout"),  #: Server timeout
     "srv-connections": TT("Connections"),  #: Server: amount of connections
+    "srv-expire_date": TT("Account expiration date"),
+    "srv-explain-expire_date": TT("Warn 5 days in advance of account expiration date."),
+    "srv-explain-quota": TT(
+        "Quota for this account, counted from the time it is set. In bytes, optionally follow with K,M,G.<br />Warn when it reaches 0, checked every few minutes."
+    ),
     "srv-retention": TT("Retention time"),  #: Server's retention time in days
     "srv-ssl": TT("SSL"),  #: Server SSL tickbox
     "explain-ssl": TT("Secure connection to server"),  #: Server SSL tickbox
@@ -622,6 +646,10 @@ SKIN_TEXT = {
     "ssl_verify-strict": TT("Strict"),
     "srv-priority": TT("Priority"),  #: Server priority
     "explain-svrprio": TT("0 is highest priority, 100 is the lowest priority"),  #: Explain server priority
+    "srv-required": TT("Required"),  #: Server required tickbox
+    "explain-required": TT(
+        "In case of connection failures, the download queue will be paused for a few minutes instead of skipping this server"
+    ),  #: Explain server required tickbox
     "srv-optional": TT("Optional"),  #: Server optional tickbox
     "explain-optional": TT(
         "For unreliable servers, will be ignored longer in case of failures"
@@ -636,6 +664,10 @@ SKIN_TEXT = {
     "srv-send_group": TT("Send Group"),
     "srv-explain-send_group": TT("Send group command before requesting articles."),
     "srv-notes": TT("Personal notes"),
+    "srv-article-availability": TT("Article availability"),
+    "srv-articles-tried": TT(
+        "%f% available of %d requested articles"
+    ),  #: Server article availability, %f=percentage, %d=number of articles
     # Config->Scheduling
     "addSchedule": TT("Add Schedule"),  #:Config->Scheduling
     "sch-frequency": TT("Frequency"),  #:Config->Scheduling
@@ -655,6 +687,8 @@ SKIN_TEXT = {
     "addMultipleFeeds": TT("Seperate multiple URLs by a comma"),  #: Config->RSS, placeholder (cannot be too long)
     "button-preFeed": TT("Read Feed"),  #: Config->RSS button
     "button-forceFeed": TT("Force Download"),  #: Config->RSS button
+    "rss-edit": TT("Edit"),  #: Config->RSS edit button
+    "rss-nextscan": TT("Next scan at"),  #: Config->RSS when will be the next RSS scan
     "rss-order": TT("Order"),  #: Config->RSS table column header
     "rss-type": TT("Type"),  #: Config->RSS table column header
     "rss-filter": TT("Filter"),  #: Config->RSS table column header
@@ -673,6 +707,9 @@ SKIN_TEXT = {
     "link-download": TT("Download"),  #: Config->RSS button "download item"
     "button-rssNow": TT("Read All Feeds Now"),  #: Config->RSS button
     # Config->Notifications
+    "defaultNotifiesAll": TT(
+        "If only the <em>Default</em> category is selected, notifications are enabled for jobs in all categories."
+    ),
     "opt-email_endjob": TT("Email Notification On Job Completion"),
     "email-never": TT("Never"),  #: When to send email
     "email-always": TT("Always"),  #: When to send email
@@ -696,7 +733,7 @@ SKIN_TEXT = {
     "opt-ncenter_enable": TT("Notification Center"),
     "opt-acenter_enable": TT("Enable Windows Notifications"),
     "testNotify": TT("Test Notification"),
-    "section-NC": TT("Notification Center"),  #: Header for OSX Notfication Center section
+    "section-NC": TT("Notification Center"),  #: Header for macOS Notfication Center section
     "section-AC": TT("Windows Notifications"),
     "section-OSD": TT("NotifyOSD"),  #: Header for Ubuntu's NotifyOSD notifications section
     "section-Prowl": TT("Prowl"),  #: Header for Prowl notification section
@@ -745,6 +782,7 @@ SKIN_TEXT = {
     "catTags": TT("Indexer Categories / Groups"),
     "button-delCat": TT("X"),  #: Small delete button
     # Config->Sorting
+    "selectOneCat": TT("Select at least 1 category."),
     "seriesSorting": TT("Series Sorting"),
     "opt-tvsort": TT("Enable TV Sorting"),
     "sort-legenda": TT("Pattern Key"),
@@ -799,6 +837,11 @@ SKIN_TEXT = {
     "button-DailyF": TT("Daily Folders"),
     "case-adjusted": TT("case-adjusted"),  #: Note for title expression in Sorting that does case adjustment
     "sortResult": TT("Processed Result"),
+    "sort-guessitMeaning": TT("Any property"),
+    "sort-guessitProperty": TT("property"),
+    "guessit-sp-property": TT("GuessIt Property"),
+    "guessit-dot-property": TT("GuessIt.Property"),
+    "guessit-us-property": TT("GuessIt_Property"),
     # Config->Special
     "explain-special": TT(
         "Rarely used options. For their meaning and explanation, click on the Help button to go to the Wiki page.<br>"
@@ -865,8 +908,8 @@ SKIN_TEXT = {
     "Glitter-queueItemLimit": TT("Queue item limit"),
     "Glitter-historyItemLimit": TT("History item limit"),
     "Glitter-dateFormat": TT("Date format"),
-    "Glitter-showExtraQueueColumn": TT("Extra queue column"),
-    "Glitter-showExtraHistoryColumn": TT("Extra history column"),
+    "Glitter-showExtraQueueColumn": TT("Extra queue columns"),
+    "Glitter-showExtraHistoryColumn": TT("Extra history columns"),
     "Glitter-page": TT("page"),
     "Glitter-loading": TT("Loading"),
     "Glitter-articles": TT("articles"),
@@ -895,9 +938,7 @@ SKIN_TEXT = {
     "Glitter-more": TT("More"),
     "Glitter-scriptLog": TT("View Script Log"),
     "Glitter-clearHistory": TT("Purge History"),
-    "Glitter-confirmClearWarnings": TT("Are you sure?"),
-    "Glitter-confirmClearDownloads": TT("Are you sure?"),
-    "Glitter-confirmClear1Download": TT("Are you sure?"),
+    "Glitter-confirmAbortDirectUnpack": TT("Renaming the job will abort Direct Unpack."),
     "Glitter-updateAvailable": TT("Update Available!"),
     "Glitter-noLocalStorage": TT(
         "LocalStorage (cookies) are disabled in your browser, interface settings will be lost after you close the browser!"
@@ -905,6 +946,7 @@ SKIN_TEXT = {
     "Glitter-glitterTips": TT("Glitter has some (new) features you might like!"),
     "Glitter-custom": TT("Custom"),
     "Glitter-displayCompact": TT("Compact layout"),
+    "Glitter-displayFullWidth": TT("Always use full screen width"),
     "Glitter-displayTabbed": TT("Tabbed layout <br/>(separate queue and history)"),
     "Glitter-speed": TT("Speed"),
     "Glitter-confirmDeleteQueue": TT("Confirm Queue Deletions"),
@@ -927,68 +969,6 @@ SKIN_TEXT = {
     "Glitter-notification-removing1": TT("Removing job"),  # Notification window
     "Glitter-notification-removing": TT("Removing jobs"),  # Notification window
     "Glitter-notification-shutdown": TT("Shutting down"),  # Notification window
-    # Plush skin
-    "Plush-confirmWithoutSavingPrompt": TT("Changes have not been saved, and will be lost."),
-    "Plush-confirm": TT("Are you sure?"),
-    "Plush-openInfoURL": TT("Open Informational URL"),
-    "Plush-viewScriptLog": TT("View Script Log"),
-    "Plush-prev": TT("Prev"),
-    "Plush-next": TT("Next"),
-    "Plush-confirmPurgeH": TT("Purge the History?"),
-    "Plush-enableJavascript": TT("You must enable JavaScript for Plush to function!"),
-    "Plush-addnzb": TT("Add NZB"),
-    "Plush-button-refresh": TT("Refresh"),
-    "Plush-options": TT("Options"),
-    "Plush-updateAvailable": TT("Update Available!"),
-    "Plush-pause5m": TT("Pause for 5 minutes"),
-    "Plush-pause15m": TT("Pause for 15 minutes"),
-    "Plush-pause30m": TT("Pause for 30 minutes"),
-    "Plush-pause1h": TT("Pause for 1 hour"),
-    "Plush-pause3h": TT("Pause for 3 hours"),
-    "Plush-pause6h": TT("Pause for 6 hours"),
-    "Plush-pauseForPrompt": TT("Pause for how many minutes?"),
-    "Plush-pauseFor": TT("Pause for..."),
-    "Plush-multiOperations": TT("Multi-Operations"),
-    "Plush-topMenu": TT("Top Menu"),
-    "Plush-onQueueFinish": TT("On Finish"),
-    "Plush-sort": TT("Sort"),
-    "Plush-sortAgeAsc": TT("Sort by Age <small>(Oldest&rarr;Newest)</small>"),
-    "Plush-sortAgeDesc": TT("Sort by Age <small>(Newest&rarr;Oldest)</small>"),
-    "Plush-sortNameAsc": TT("Sort by Name <small>(A&rarr;Z)</small>"),
-    "Plush-sortNameDesc": TT("Sort by Name <small>(Z&rarr;A)</small>"),
-    "Plush-sortSizeAsc": TT("Sort by Size <small>(Smallest&rarr;Largest)</small>"),
-    "Plush-sortSizeDesc": TT("Sort by Size <small>(Largest&rarr;Smallest)</small>"),
-    "Plush-confirmPurgeQ": TT("Purge the Queue?"),
-    "Plush-confirmRetryQ": TT("Retry all failed jobs in History?"),
-    "Plush-purge": TT("Purge"),
-    "Plush-left": TT("left"),
-    "Plush-maxSpeed": TT("Max Speed"),  #: Used in speed menu. Split in two lines if too long.
-    "Plush-nzo-range": TT("Range"),
-    "Plush-reset": TT("Reset"),
-    "Plush-applySelected": TT("Apply to Selected"),
-    "Plush-page": TT("page"),
-    "Plush-everything": TT("Everything"),
-    "Plush-disabled": TT("Disabled"),
-    "Plush-refreshRate": TT("Refresh Rate"),
-    "Plush-containerWidth": TT("Container Width"),
-    "Plush-confirmDeleteQueue": TT("Confirm Queue Deletions"),
-    "Plush-confirmDeleteHistory": TT("Confirm History Deletions"),
-    "Plush-explain-blockRefresh": TT(
-        "This will prevent refreshing content when your mouse cursor is hovering over the queue."
-    ),
-    "Plush-blockRefresh": TT("Block Refreshes on Hover"),
-    "Plush-fetch": TT("Fetch"),  #: Fetch from URL button in "Add NZB" dialog box
-    "Plush-upload": TT("Upload"),  #: Upload button in "Add NZB" dialog box
-    "Plush-uploadTip": TT("Upload: .nzb .rar .zip .gz, .bz2"),
-    "Plush-addnzb-filename": TT("Optionally specify a filename"),
-    "Plush-progress": TT("Progress"),
-    "Plush-remaining": TT("Remaining"),
-    "Plush-notEnoughSpace": TT("Not enough disk space to complete downloads!"),
-    "Plush-freeSpace": TT("Free Space"),
-    "Plush-freeSpaceTemp": TT("Free (Temp)"),
-    "Plush-idle": TT("IDLE"),
-    "Plush-downloads": TT("Downloads"),
-    "Plush-tab-repair": TT("Queue repair"),
     # Wizard
     "wizard-quickstart": TT("SABnzbd Quick-Start Wizard"),
     "wizard-version": TT("SABnzbd Version"),
