@@ -1,5 +1,5 @@
 #!/usr/bin/python3 -OO
-# Copyright 2007-2020 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2022 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -64,7 +64,7 @@ def test_nntp_server_dict(kwargs):
 
 
 def test_nntp_server(host, port, server=None, username=None, password=None, ssl=None, ssl_verify=1, ssl_ciphers=None):
-    """ Will connect (blocking) to the nttp server and report back any errors """
+    """Will connect (blocking) to the nttp server and report back any errors"""
     timeout = 4.0
     if "*" in password and not password.strip("*"):
         # If the password is masked, try retrieving it from the config
@@ -90,7 +90,7 @@ def test_nntp_server(host, port, server=None, username=None, password=None, ssl=
 
     try:
         nw = NewsWrapper(s, -1, block=True)
-        nw.init_connect(None)
+        nw.init_connect()
         while not nw.connected:
             nw.clear_data()
             nw.recv_chunk(block=True)
@@ -108,7 +108,7 @@ def test_nntp_server(host, port, server=None, username=None, password=None, ssl=
 
     except socket.error as e:
         # Trying SSL on non-SSL port?
-        if "unknown protocol" in str(e).lower():
+        if "unknown protocol" in str(e).lower() or "wrong version number" in str(e).lower():
             return False, T("Unknown SSL protocol: Try disabling SSL or connecting on a different port.")
 
         return False, str(e)
@@ -132,20 +132,17 @@ def test_nntp_server(host, port, server=None, username=None, password=None, ssl=
             # Some internal error, not always safe to close connection
             return False, str(sys.exc_info()[1])
 
-    # Close the connection
-    nw.terminate(quit=True)
-
     if nw.status_code == 480:
-        return False, T("Server requires username and password.")
-
+        return_status = (False, T("Server requires username and password."))
     elif nw.status_code == 100 or str(nw.status_code).startswith(("2", "4")):
-        return True, T("Connection Successful!")
-
+        return_status = (True, T("Connection Successful!"))
     elif nw.status_code == 502 or clues_login(nntp_to_msg(nw.data)):
-        return False, T("Authentication failed, check username/password.")
-
+        return_status = (False, T("Authentication failed, check username/password."))
     elif clues_too_many(nntp_to_msg(nw.data)):
-        return False, T("Too many connections, please pause downloading or try again later")
-
+        return_status = (False, T("Too many connections, please pause downloading or try again later"))
     else:
-        return False, T("Could not determine connection result (%s)") % nntp_to_msg(nw.data)
+        return_status = (False, T("Could not determine connection result (%s)") % nntp_to_msg(nw.data))
+
+    # Close the connection and return result
+    nw.hard_reset(send_quit=True)
+    return return_status
