@@ -1,5 +1,5 @@
 #!/usr/bin/python3 -OO
-# Copyright 2007-2021 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2022 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -138,13 +138,14 @@ async def serve_sabnews(hostname, port):
     return server
 
 
-def create_nzb(nzb_file=None, nzb_dir=None):
+def create_nzb(nzb_file=None, nzb_dir=None, metadata=None):
     article_size = 500000
     files_for_nzb = []
     output_file = ""
 
     # Either use directory or single file
     if nzb_dir:
+        nzb_dir = os.path.normpath(nzb_dir)
         if not os.path.exists(nzb_dir) or not os.path.isdir(nzb_dir):
             raise NotADirectoryError("%s is not a valid directory" % nzb_dir)
 
@@ -168,6 +169,12 @@ def create_nzb(nzb_file=None, nzb_dir=None):
         nzb.write('<!DOCTYPE nzb PUBLIC "-//newzBin//DTD NZB 1.0//EN" "http://www.newzbin.com/DTD/nzb/nzb-1.0.dtd">\n')
         nzb.write('<nzb xmlns="http://www.newzbin.com/DTD/2003/nzb">\n')
 
+        if metadata:
+            nzb.write("<head>\n")
+            for meta_name, meta_value in metadata.items():
+                nzb.write('<meta type="%s">%s</meta>\n' % (meta_name, meta_value))
+            nzb.write("</head>\n")
+
         nzb_time = time.time() - randint(0, int(time.time() - 746863566))
 
         for fl in files_for_nzb:
