@@ -1,5 +1,5 @@
 #!/usr/bin/python3 -OO
-# Copyright 2007-2020 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2022 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -20,12 +20,16 @@ tests.test_filesystem - Testing functions in filesystem.py
 """
 import stat
 import sys
+import os
+import random
+import shutil
+from pathlib import Path
 
 import pyfakefs.fake_filesystem_unittest as ffs
 
 import sabnzbd.cfg
 import sabnzbd.filesystem as filesystem
-from sabnzbd.constants import DEF_FOLDER_MAX
+from sabnzbd.constants import DEF_FOLDER_MAX, DEF_FILE_MAX
 from tests.testhelper import *
 
 # Set the global uid for fake filesystems to a non-root user;
@@ -60,7 +64,6 @@ class TestFileFolderNameSanitizer:
         assert filesystem.sanitize_filename("test: ") == "unknown"
 
     @set_platform("linux")
-    @set_config({"sanitize_safe": False})
     def test_colon_handling_other(self):
         assert filesystem.sanitize_filename("test:aftertest") == "test:aftertest"
         assert filesystem.sanitize_filename(":") == ":"
@@ -76,7 +79,6 @@ class TestFileFolderNameSanitizer:
         assert filesystem.sanitize_filename("a$mft") == "a$mft"
 
     @set_platform("linux")
-    @set_config({"sanitize_safe": False})
     def test_win_devices_not_win(self):
         # Linux and Darwin are the same for this
         assert filesystem.sanitize_filename(None) is None
@@ -86,7 +88,6 @@ class TestFileFolderNameSanitizer:
         assert filesystem.sanitize_filename("a$mft") == "a$mft"
 
     @set_platform("linux")
-    @set_config({"sanitize_safe": False})
     def test_file_illegal_chars_linux(self):
         assert filesystem.sanitize_filename("test/aftertest") == "test+aftertest"
         assert filesystem.sanitize_filename("/test") == "+test"
@@ -98,7 +99,6 @@ class TestFileFolderNameSanitizer:
         assert filesystem.sanitize_filename("../test") == "..+test"
 
     @set_platform("linux")
-    @set_config({"sanitize_safe": False})
     def test_folder_illegal_chars_linux(self):
         assert filesystem.sanitize_foldername('test"aftertest') == "test'aftertest"
         assert filesystem.sanitize_foldername("test:") == "test-"
@@ -109,7 +109,6 @@ class TestFileFolderNameSanitizer:
         assert len(filesystem.CH_ILLEGAL_WIN) == len(filesystem.CH_LEGAL_WIN)
 
     @set_platform("linux")
-    @set_config({"sanitize_safe": False})
     def test_legal_chars_linux(self):
         # Illegal on Windows but not on Linux, unless sanitize_safe is active.
         # Don't bother with '/' which is illegal in filenames on all platforms.
@@ -162,8 +161,10 @@ class TestFileFolderNameSanitizer:
         assert filesystem.sanitize_foldername("test..aftertest") == "test..aftertest"
         assert filesystem.sanitize_foldername("test.aftertest.") == "test.aftertest"
         assert filesystem.sanitize_foldername("test.aftertest..") == "test.aftertest"
+        assert filesystem.sanitize_foldername("test. aftertest. . . .") == "test. aftertest"
         assert filesystem.sanitize_foldername("/test/this.") == "+test+this"
         assert filesystem.sanitize_foldername("/test./this.") == "+test.+this"
+        assert filesystem.sanitize_foldername("/test. /this . ") == "+test. +this"
 
     def test_long_foldername(self):
         assert len(filesystem.sanitize_foldername("test" * 100)) == DEF_FOLDER_MAX
@@ -187,6 +188,108 @@ class TestFileFolderNameSanitizer:
         assert filesystem.sanitize_foldername("\t\t\t") == "unknown"
         assert filesystem.sanitize_foldername(" ") == "unknown"
         assert filesystem.sanitize_foldername("  ") == "unknown"
+        assert filesystem.sanitize_foldername(" . .") == "unknown"
+
+    def test_filename_too_long(self):
+
+        # Note: some filesystem can handle up to 255 UTF chars (which is more than 255 bytes) in the filename,
+        # but we stay on the safe side: max DEF_FILE_MAX bytes
+
+        # PART 1: Base cases: Nothing should happen:
+
+        # normal filename
+        name = "a" * 200 + ".ext"
+        sanitizedname = filesystem.sanitize_filename(name)
+        assert sanitizedname == name
+
+        # Unicode / UTF8 is OK ... as total filename length is not too long
+        name = "BASE" + "你" * 50 + "blabla.ext"
+        sanitizedname = filesystem.sanitize_filename(name)
+        assert sanitizedname == name
+
+        # filename with very long extension, but total filename is no problem, so no change
+        name = "hello.ext" + "e" * 200
+        sanitizedname = filesystem.sanitize_filename(name)
+        assert sanitizedname == name  # no change
+
+        # PART 2: base truncating
+
+        name = "BASE" + "a" * 300 + ".mylongext"
+        sanitizedname = filesystem.sanitize_filename(name)
+        assert len(sanitizedname) <= DEF_FILE_MAX
+        assert sanitizedname.startswith("BASEaaaaaaaaaaaaaaa")
+        assert sanitizedname.endswith(".mylongext")
+
+        # too long filename, so truncate keeping the start of name and ext should stay the same
+        name = "BASE" + "a" * 200 + ".EXT" + "e" * 200
+        sanitizedname = filesystem.sanitize_filename(name)
+        assert len(sanitizedname) <= DEF_FILE_MAX
+        newname, newext = os.path.splitext(sanitizedname)
+        assert newname.startswith("BASEaaaaa")
+        assert newext.startswith(".EXTeeeee")
+
+        # PART 3: more exotic cases
+
+        # insert NON-ASCII chars, which should stay in place because overall length is no problem
+        name = "aaaa" + 10 * chr(188) + 10 * chr(222) + "bbbb.ext"
+        sanitizedname = filesystem.sanitize_filename(name)
+        assert sanitizedname == name
+
+        # insert NON-ASCII chars, which should get removed because overall length is too long
+        name = "aaaa" + 200 * chr(188) + 200 * chr(222) + "bbbb.ext"
+        sanitizedname = filesystem.sanitize_filename(name)
+        assert sanitizedname == "aaaabbbb.ext"
+
+        # Unicode / UTF8 ... total filename length might be too long for certain filesystems
+        name = "BASE" + "你" * 200 + ".ext"
+        sanitizedname = filesystem.sanitize_filename(name)
+        assert sanitizedname.startswith("BASE")
+        assert sanitizedname.endswith(".ext")
+
+        # Linux / POSIX: a hidden file (no extension), with size 200, so do not truncate at all
+        name = "." + "a" * 200
+        sanitizedname = filesystem.sanitize_filename(name)
+        assert sanitizedname == name  # no change
+
+
+class TestSanitizeFiles(ffs.TestCase):
+    def setUp(self):
+        self.setUpPyfakefs()
+        self.fs.path_separator = "\\"
+        self.fs.is_windows_fs = True
+        # Disable randomisation of directory listings
+        self.fs.shuffle_listdir_results = False
+
+    def test_sanitize_files_input(self):
+        assert [] == filesystem.sanitize_files(folder=None)
+        assert [] == filesystem.sanitize_files(filelist=None)
+        assert [] == filesystem.sanitize_files(folder=None, filelist=None)
+
+    @set_platform("win32")
+    @set_config({"sanitize_safe": True})
+    def test_sanitize_files(self):
+        # The very specific tests of sanitize_filename() are above
+        # Here we just want to see that sanitize_files() works as expected
+        input_list = [r"c:\test\con.man", r"c:\test\foo:bar"]
+        output_list = [r"c:\test\_con.man", r"c:\test\foo-bar"]
+
+        # Test both the "folder" and "filelist" based calls
+        for kwargs in ({"folder": r"c:\test"}, {"filelist": input_list}):
+            # Create source files
+            for file in input_list:
+                self.fs.create_file(file)
+
+            assert output_list == filesystem.sanitize_files(**kwargs)
+
+            # Make sure the old ones are gone
+            for file in input_list:
+                assert not os.path.exists(file)
+
+            # Make sure the new ones are there
+            for file in output_list:
+                assert os.path.exists(file)
+                os.remove(file)
+                assert not os.path.exists(file)
 
 
 class TestSameFile:
@@ -241,28 +344,6 @@ class TestSameFile:
         assert 0 == filesystem.same_file("/test/../Home", "/home")
 
 
-class TestIsObfuscatedFilename:
-    def test_obfuscated(self):
-        # Files are considered obfuscated if they lack an extension
-        assert filesystem.is_obfuscated_filename(".") is True
-        assert filesystem.is_obfuscated_filename("..") is True
-        assert filesystem.is_obfuscated_filename(".test") is True
-        assert filesystem.is_obfuscated_filename("test.") is True
-        assert filesystem.is_obfuscated_filename("test.ext.") is True
-        assert filesystem.is_obfuscated_filename("t.....") is True
-        assert filesystem.is_obfuscated_filename("a_" + ("test" * 666)) is True
-
-    def test_not_obfuscated(self):
-        assert filesystem.is_obfuscated_filename("test.ext") is False
-        assert filesystem.is_obfuscated_filename(".test.ext") is False
-        assert filesystem.is_obfuscated_filename("test..ext") is False
-        assert filesystem.is_obfuscated_filename("test.ext") is False
-        assert filesystem.is_obfuscated_filename("test .ext") is False
-        assert filesystem.is_obfuscated_filename("test. ext") is False
-        assert filesystem.is_obfuscated_filename("test . ext") is False
-        assert filesystem.is_obfuscated_filename("a." + ("test" * 666)) is False
-
-
 class TestClipLongPath:
     def test_empty(self):
         assert filesystem.clip_path(None) is None
@@ -317,10 +398,10 @@ class TestCheckMountLinux(ffs.TestCase):
         self.setUpPyfakefs()
         self.fs.path_separator = "/"
         self.fs.is_case_sensitive = True
-        for dir in self.test_dirs:
-            self.fs.create_dir(dir, perm_bits=755)
+        for test_dir in self.test_dirs:
+            self.fs.create_dir(test_dir, perm_bits=755)
             # Sanity check the fake filesystem
-            assert os.path.exists(dir) is True
+            assert os.path.exists(test_dir) is True
 
     @set_platform("linux")
     def test_bare_mountpoint_linux(self):
@@ -451,19 +532,6 @@ class TestCheckMountWin(ffs.TestCase):
         assert filesystem.check_mount("//test/that/") is True
 
 
-class TestTrimWinPath:
-    @set_platform("win32")
-    def test_short_path(self):
-        assert filesystem.trim_win_path(r"C:\short\path") == r"C:\short\path"
-
-    @pytest.mark.xfail(sys.platform == "win32", reason="Bug in trim_win_path")
-    @set_platform("win32")
-    def test_long_path_short_segments(self):
-        test_path = "C:\\" + "A" * 20 + "\\" + "B" * 20 + "\\" + "C" * 20  # Strlen 65
-        # Current code causes the path to end up with strlen 70 rather than 69 on Windows
-        assert filesystem.trim_win_path(test_path + "\\" + ("D" * 20)) == test_path + "\\" + "D" * 3
-
-
 @pytest.mark.skipif(sys.platform.startswith("win"), reason="Non-Windows tests")
 class TestListdirFull(ffs.TestCase):
     # Basic fake filesystem setup stanza
@@ -623,7 +691,7 @@ class TestListdirFullWin(ffs.TestCase):
 
 
 @pytest.mark.skipif(sys.platform.startswith("win"), reason="Non-Windows tests")
-class TestGetUniquePathFilename(ffs.TestCase):
+class TestGetUniqueDirFilename(ffs.TestCase):
     # Basic fake filesystem setup stanza
     def setUp(self):
         self.setUpPyfakefs()
@@ -634,25 +702,25 @@ class TestGetUniquePathFilename(ffs.TestCase):
     @set_config({"wait_ext_drive": 1})
     def test_nonexistent_dir(self):
         # Absolute path
-        assert filesystem.get_unique_path("/foo/bar", n=0, create_dir=False) == "/foo/bar"
+        assert filesystem.get_unique_dir("/foo/bar", n=0, create_dir=False) == "/foo/bar"
         # Absolute path in a location that matters to check_mount
-        assert filesystem.get_unique_path("/mnt/foo/bar", n=0, create_dir=False) == "/mnt/foo/bar"
+        assert filesystem.get_unique_dir("/mnt/foo/bar", n=0, create_dir=False) == "/mnt/foo/bar"
         # Relative path
         if self.fs.cwd != "/":
             os.chdir("/")
-        assert filesystem.get_unique_path("foo/bar", n=0, create_dir=False) == "foo/bar"
+        assert filesystem.get_unique_dir("foo/bar", n=0, create_dir=False) == "foo/bar"
 
     def test_creating_dir(self):
         # First call also creates the directory for us
-        assert filesystem.get_unique_path("/foo/bar", n=0, create_dir=True) == "/foo/bar"
+        assert filesystem.get_unique_dir("/foo/bar", n=0, create_dir=True) == "/foo/bar"
         # Verify creation of the path
         assert os.path.exists("/foo/bar") is True
         # Directories from previous loops get in the way
         for dir_n in range(1, 11):  # Go high enough for double digits
-            assert filesystem.get_unique_path("/foo/bar", n=0, create_dir=True) == "/foo/bar." + str(dir_n)
+            assert filesystem.get_unique_dir("/foo/bar", n=0, create_dir=True) == "/foo/bar." + str(dir_n)
             assert os.path.exists("/foo/bar." + str(dir_n)) is True
         # Explicitly set parameter n
-        assert filesystem.get_unique_path("/foo/bar", n=666, create_dir=True) == "/foo/bar.666"
+        assert filesystem.get_unique_dir("/foo/bar", n=666, create_dir=True) == "/foo/bar.666"
         assert os.path.exists("/foo/bar.666") is True
 
     def test_nonexistent_file(self):
@@ -677,11 +745,14 @@ class TestGetUniquePathFilename(ffs.TestCase):
         # Create obstructions
         self.fs.create_file(test_file)
         assert os.path.exists(test_file)
-        assert filesystem.get_unique_filename(test_file) == "/some/filename.1"
+        first_filename = filesystem.get_unique_filename(test_file)
+        assert first_filename == "/some/filename.1"
+        self.fs.create_file(first_filename)
+        assert filesystem.get_unique_filename(test_file) == "/some/filename.2"
 
 
 @pytest.mark.skipif(not sys.platform.startswith("win"), reason="Windows specific tests")
-class TestGetUniquePathFilenameWin(ffs.TestCase):
+class TestGetUniqueDirFilenameWin(ffs.TestCase):
     # Basic fake filesystem setup stanza
     def setUp(self):
         self.setUpPyfakefs()
@@ -693,21 +764,21 @@ class TestGetUniquePathFilenameWin(ffs.TestCase):
     @set_config({"wait_ext_drive": 1})
     def test_nonexistent_dir(self):
         # Absolute path
-        assert filesystem.get_unique_path(r"C:\No\Such\Dir", n=0, create_dir=False).lower() == r"c:\no\such\dir"
+        assert filesystem.get_unique_dir(r"C:\No\Such\Dir", n=0, create_dir=False).lower() == r"c:\no\such\dir"
         # Relative path
-        assert filesystem.get_unique_path(r"foo\bar", n=0, create_dir=False).lower() == r"foo\bar"
+        assert filesystem.get_unique_dir(r"foo\bar", n=0, create_dir=False).lower() == r"foo\bar"
 
     def test_creating_dir(self):
         # First call also creates the directory for us
-        assert filesystem.get_unique_path(r"C:\foo\BAR", n=0, create_dir=True).lower() == r"c:\foo\bar"
+        assert filesystem.get_unique_dir(r"C:\foo\BAR", n=0, create_dir=True).lower() == r"c:\foo\bar"
         # Verify creation of the path
         assert os.path.exists(r"c:\foo\bar") is True
         # Directories from previous loops get in the way
         for dir_n in range(1, 11):  # Go high enough for double digits
-            assert filesystem.get_unique_path(r"c:\foo\bar", n=0, create_dir=True) == r"c:\foo\bar." + str(dir_n)
+            assert filesystem.get_unique_dir(r"c:\foo\bar", n=0, create_dir=True) == r"c:\foo\bar." + str(dir_n)
             assert os.path.exists(r"c:\foo\bar." + str(dir_n)) is True
         # Explicitly set parameter n
-        assert filesystem.get_unique_path(r"c:\Foo\Bar", n=666, create_dir=True).lower() == r"c:\foo\bar.666"
+        assert filesystem.get_unique_dir(r"c:\Foo\Bar", n=666, create_dir=True).lower() == r"c:\foo\bar.666"
         assert os.path.exists(r"c:\foo\bar.666") is True
 
     def test_nonexistent_file(self):
@@ -745,7 +816,7 @@ class TestCreateAllDirsWin(ffs.TestCase):
 
     @set_platform("win32")
     def test_create_all_dirs(self):
-        self.dir = self.fs.create_dir(r"C:\Downloads")
+        self.directory = self.fs.create_dir(r"C:\Downloads")
         # Also test for no crash when folder already exists
         for folder in (r"C:\Downloads", r"C:\Downloads\Show\Test", r"C:\Downloads\Show\Test2", r"C:\Downloads\Show"):
             assert filesystem.create_all_dirs(folder) == folder
@@ -772,34 +843,33 @@ class TestCreateAllDirs(ffs.TestCase, PermissionCheckerHelper):
             assert filesystem.create_all_dirs(folder) == folder
             assert os.path.exists(folder)
 
-    @set_config({"umask": "0777"})
+    @set_config({"permissions": "0777"})
     def test_permissions_777(self):
         self._permissions_runner("/test_base777")
-        self._permissions_runner("/test_base777_nomask", apply_umask=False)
+        self._permissions_runner("/test_base777_nomask", apply_permissions=False)
 
-    @set_config({"umask": "0770"})
+    @set_config({"permissions": "0770"})
     def test_permissions_770(self):
         self._permissions_runner("/test_base770")
-        self._permissions_runner("/test_base770_nomask", apply_umask=False)
+        self._permissions_runner("/test_base770_nomask", apply_permissions=False)
 
-    @set_config({"umask": "0600"})
+    @set_config({"permissions": "0600"})
     def test_permissions_600(self):
         self._permissions_runner("/test_base600")
-        self._permissions_runner("/test_base600_nomask", apply_umask=False)
+        self._permissions_runner("/test_base600_nomask", apply_permissions=False)
 
-    @set_config({"umask": "0700"})
+    @set_config({"permissions": "0700"})
     def test_permissions_450(self):
         with pytest.raises(OSError):
             self._permissions_runner("/test_base450", perms_base="0450")
 
-    @set_config({"umask": ""})
-    def test_no_umask(self):
+    def test_no_permissions(self):
         self._permissions_runner("/test_base_perm700", perms_base="0700")
         self._permissions_runner("/test_base_perm750", perms_base="0750")
         self._permissions_runner("/test_base_perm777", perms_base="0777")
         self._permissions_runner("/test_base_perm600", perms_base="0600")
 
-    def _permissions_runner(self, test_base, perms_base="0700", apply_umask=True):
+    def _permissions_runner(self, test_base, perms_base="0700", apply_permissions=True):
         # Create base directory and set the base permissions
         perms_base_int = int(perms_base, 8)
         self.fs.create_dir(test_base, perms_base_int)
@@ -808,17 +878,15 @@ class TestCreateAllDirs(ffs.TestCase, PermissionCheckerHelper):
 
         # Create directories with permissions
         new_dir = os.path.join(test_base, "se 1", "ep1")
-        filesystem.create_all_dirs(new_dir, apply_umask=apply_umask)
+        filesystem.create_all_dirs(new_dir, apply_permissions=apply_permissions)
 
         # If permissions needed to be set, verify the new folder has the
         # right permissions and verify the base didn't change
-        if apply_umask and cfg.umask():
-            perms_test_int = int(cfg.umask(), 8) | int("0700", 8)
+        if apply_permissions and cfg.permissions():
+            perms_test_int = int(cfg.permissions(), 8)
         else:
-            # Get the current umask, since os.mkdir masks that out
-            cur_umask = os.umask(0)
-            os.umask(cur_umask)
-            perms_test_int = int("0777", 8) & ~cur_umask
+            # Get the current permissions, since os.mkdir masks that out
+            perms_test_int = int("0777", 8) & ~sabnzbd.ORG_UMASK
         self.assert_dir_perms(new_dir, perms_test_int)
         self.assert_dir_perms(test_base, perms_base_int)
 
@@ -839,32 +907,28 @@ class TestSetPermissions(ffs.TestCase, PermissionCheckerHelper):
         self.fs.is_case_sensitive = True
         self.fs.umask = int("0755", 8)  # rwxr-xr-x
 
-    def _runner(self, perms_test, perms_after):
+    def _runner(self, perms_before_test):
         """
-        Generic test runner for permissions testing. The umask is set per test
+        Generic test runner for permissions testing. The permissions are set per test
         via the relevant sab config option; the fileystem parameter in setUp().
         Note that the umask set in the environment before starting the program
-        also affects the results if sabnzbd.cfg.umask isn't set.
+        also affects the results if sabnzbd.cfg.permissions isn't set.
 
         Arguments:
             str perms_test: permissions for test objects, chmod style "0755".
-            str perms_after: expected permissions after completion of the test.
         """
-        perms_test = int(perms_test, 8)
-        if sabnzbd.cfg.umask():
-            perms_after = int(perms_after, 8)
+        # We expect the cfg.permissions to be applied, or the original to be kept if none are set
+        perms_before_test = int(perms_before_test, 8)
+        if sabnzbd.cfg.permissions():
+            perms_after_test = int(sabnzbd.cfg.permissions(), 8)
         else:
-            perms_after = int("0777", 8) & (sabnzbd.ORG_UMASK ^ int("0777", 8))
+            perms_after_test = perms_before_test
 
         # Setup and verify fake dir
         test_dir = "/test"
-        try:
-            self.fs.create_dir(test_dir, perms_test)
-        except PermissionError:
-            ffs.set_uid(0)
-            self.fs.create_dir(test_dir, perms_test)
+        self.fs.create_dir(test_dir, perms_before_test)
         assert os.path.exists(test_dir) is True
-        self.assert_dir_perms(test_dir, perms_test)
+        self.assert_dir_perms(test_dir, perms_before_test)
 
         # Setup and verify fake files
         for file in (
@@ -874,32 +938,47 @@ class TestSetPermissions(ffs.TestCase, PermissionCheckerHelper):
             "another/sub/dir/WithSome.File",
         ):
             file = os.path.join(test_dir, file)
-            try:
-                self.fs.create_file(file, perms_test)
-            except PermissionError:
+            basefolder = os.path.dirname(file)
+
+            # Create the folder, so it has the expected permissions
+            if not os.path.exists(basefolder):
                 try:
+                    self.fs.create_dir(basefolder, perms_before_test)
+                except PermissionError:
                     ffs.set_uid(0)
-                    self.fs.create_file(file, perms_test)
-                except Exception:
-                    # Skip creating files, if not even using root gets the job done.
-                    break
+                    self.fs.create_file(file, perms_before_test)
+            assert os.path.exists(basefolder) is True
+            self.assert_dir_perms(basefolder, perms_before_test)
+
+            # Add a random one of the forbidden bits
+            file_perms_before_test = perms_before_test | choice(
+                (stat.S_ISUID, stat.S_ISGID, stat.S_IXUSR, stat.S_IXGRP, stat.S_IXOTH)
+            )
+
+            # Then, create the file
+            try:
+                self.fs.create_file(file, file_perms_before_test)
+            except PermissionError:
+                ffs.set_uid(0)
+                self.fs.create_file(file, file_perms_before_test)
+
             assert os.path.exists(file) is True
-            assert stat.filemode(os.stat(file).st_mode)[1:] == stat.filemode(perms_test)[1:]
+            assert stat.filemode(os.stat(file).st_mode)[1:] == stat.filemode(file_perms_before_test)[1:]
 
         # Set permissions, recursive by default
         filesystem.set_permissions(test_dir)
 
         # Check the results
         for root, dirs, files in os.walk(test_dir):
-            for dir in [os.path.join(root, d) for d in dirs]:
+            for directory in [os.path.join(root, d) for d in dirs]:
                 # Permissions on directories should now match perms_after
-                self.assert_dir_perms(dir, perms_after)
+                self.assert_dir_perms(directory, perms_after_test)
             for file in [os.path.join(root, f) for f in files]:
                 # Files also shouldn't have any executable or special bits set
                 assert (
                     stat.filemode(os.stat(file).st_mode)[1:]
                     == stat.filemode(
-                        perms_after & ~(stat.S_ISUID | stat.S_ISGID | stat.S_IXUSR | stat.S_IXGRP | stat.S_IXOTH)
+                        perms_after_test & ~(stat.S_ISUID | stat.S_ISGID | stat.S_IXUSR | stat.S_IXGRP | stat.S_IXOTH)
                     )[1:]
                 )
 
@@ -910,49 +989,168 @@ class TestSetPermissions(ffs.TestCase, PermissionCheckerHelper):
         ffs.set_uid(global_uid)
 
     @set_platform("linux")
-    @set_config({"umask": ""})
-    def test_dir0777_empty_umask_setting(self):
+    def test_empty_permissions_setting(self):
         # World writable directory
-        self._runner("0777", "0700")
+        self._runner("0777")
+        self._runner("0450")
 
     @set_platform("linux")
-    @set_config({"umask": ""})
-    def test_dir0450_empty_umask_setting(self):
-        # Insufficient access
-        self._runner("0450", "0700")
+    @set_config({"permissions": "0760"})
+    def test_dir0777_permissions0760_setting(self):
+        # World-writable directory, permissions 760
+        self._runner("0777")
 
     @set_platform("linux")
-    @set_config({"umask": ""})
-    def test_dir0000_empty_umask_setting(self):
-        # Weird directory permissions
-        self._runner("0000", "0700")
+    @set_config({"permissions": "0617"})
+    def test_dir0450_permissions0617_setting(self):
+        # Insufficient base access
+        self._runner("0450")
 
     @set_platform("linux")
-    @set_config({"umask": "0760"})
-    def test_dir0777_umask0760_setting(self):
-        # World-writable directory, umask 760
-        self._runner("0777", "0760")
+    @set_config({"permissions": "2455"})
+    def test_dir0444_permissions2455_setting(self):
+        # Insufficient access, permissions with setgid (should be stripped)
+        self._runner("0444")
 
     @set_platform("linux")
-    @set_config({"umask": "0617"})
-    def test_dir0450_umask0617_setting(self):
-        # Insufficient access, weird umask
-        self._runner("0450", "0717")
+    @set_config({"permissions": "4755"})
+    def test_dir1755_permissions4755_setting(self):
+        # Sticky bit on directory, permissions with setuid (should be stripped)
+        self._runner("1755")
 
-    @set_platform("linux")
-    @set_config({"umask": "0000"})
-    def test_dir0405_umask0000_setting(self):
-        # Insufficient access on all fronts, weird umask
-        self._runner("0405", "0700")
 
-    @set_platform("linux")
-    @set_config({"umask": "2455"})
-    def test_dir0444_umask2455_setting(self):
-        # Insufficient access, weird umask with setgid
-        self._runner("0444", "2755")
+class TestRenamer:
+    # test filesystem.renamer() for different scenario's
+    def test_renamer(self):
+        # First of all, create a working directory (with a random name)
+        dirname = os.path.join(SAB_DATA_DIR, "testdir" + str(random.randint(10000, 99999)))
+        os.mkdir(dirname)
 
-    @set_platform("linux")
-    @set_config({"umask": "4755"})
-    def test_dir1755_umask4755_setting(self):
-        # Sticky bit on directory, umask with setuid
-        self._runner("1755", "4755")
+        # base case: rename file within directory
+        filename = os.path.join(dirname, "myfile.txt")
+        Path(filename).touch()  # create file
+        newfilename = os.path.join(dirname, "newfile.txt")
+        assert newfilename == filesystem.renamer(filename, newfilename)
+        assert not os.path.isfile(filename)
+        assert os.path.isfile(newfilename)
+
+        # standard behaviour: renaming (moving) into an exiting other directory *is* allowed
+        filename = os.path.join(dirname, "myfile.txt")
+        Path(filename).touch()  # create file
+        sameleveldirname = os.path.join(SAB_DATA_DIR, "othertestdir" + str(random.randint(10000, 99999)))
+        os.mkdir(sameleveldirname)
+        newfilename = os.path.join(sameleveldirname, "newfile.txt")
+        assert newfilename == filesystem.renamer(filename, newfilename)
+        assert not os.path.isfile(filename)
+        assert os.path.isfile(newfilename)
+        shutil.rmtree(sameleveldirname)
+
+        # Default: renaming into a non-existing subdirectory not allowed
+        Path(filename).touch()  # create file
+        newfilename = os.path.join(dirname, "nonexistingsubdir", "newfile.txt")
+        try:
+            # Should fail
+            filesystem.renamer(filename, newfilename)
+        except:
+            pass
+        assert os.path.isfile(filename)
+        assert not os.path.isfile(newfilename)
+
+        # Creation of subdirectory is allowed if create_local_directories=True
+        Path(filename).touch()
+        newfilename = os.path.join(dirname, "newsubdir", "newfile.txt")
+        try:
+            filesystem.renamer(filename, newfilename, create_local_directories=True)
+        except:
+            pass
+        assert not os.path.isfile(filename)
+        assert os.path.isfile(newfilename)
+
+        # Creation of subdirectory plus deeper sudbdir is allowed if create_local_directories=True
+        Path(filename).touch()
+        newfilename = os.path.join(dirname, "newsubdir", "deepersubdir", "newfile.txt")
+        try:
+            filesystem.renamer(filename, newfilename, create_local_directories=True)
+        except:
+            pass
+        assert not os.path.isfile(filename)
+        assert os.path.isfile(newfilename)
+
+        # ... escaping the directory plus subdir creation is not allowed
+        Path(filename).touch()
+        newfilename = os.path.join(dirname, "..", "newsubdir", "newfile.txt")
+        try:
+            filesystem.renamer(filename, newfilename, create_local_directories=True)
+        except:
+            pass
+        assert os.path.isfile(filename)
+        assert not os.path.isfile(newfilename)
+
+        # Cleanup working directory
+        shutil.rmtree(dirname)
+
+
+class TestUnwantedExtensions:
+    # Only test lowercase extensions without a leading dot: the unwanted_extensions
+    # setting is sanitized accordingly in interface.saveSwitches() before saving.
+    test_extensions = "iso, cmd, bat, sh, re:r[0-9]{2}, sab*"
+    # Test parameters as (filename, result) tuples, with result given for blacklist mode
+    test_params = [
+        ("ubuntu.iso", True),
+        ("par2.cmd", True),
+        ("freedos.BAT", True),
+        ("Debian.installer.SH", True),
+        ("FREEBSD.ISO", True),
+        ("par2.CmD", True),
+        ("freedos.baT", True),
+        ("Debian.Installer.sh", True),
+        ("ubuntu.torrent", False),
+        ("par2.cmd.notcmd", False),
+        ("freedos.tab", False),
+        (".SH.hs", False),
+        ("regexp.r0611", False),
+        ("regexp.007", False),
+        ("regexp.A01", False),
+        ("regexp.r9", False),
+        ("regexp.r2d2", False),
+        ("regexp.r2d", False),
+        ("regexp.r00", True),
+        ("regexp.R42", True),
+        ("test.sabnzbd", True),
+        ("pass.sab", True),
+        ("fail.sb", False),
+        ("No_Extension", False),
+        ("r42", False),
+        (480, False),
+        (None, False),
+        ("", False),
+        ([], False),
+    ]
+
+    @set_config({"unwanted_extensions_mode": 0, "unwanted_extensions": test_extensions})
+    def test_has_unwanted_extension_blacklist_mode(self):
+        for filename, result in self.test_params:
+            assert filesystem.has_unwanted_extension(filename) is result
+
+    @set_config({"unwanted_extensions_mode": 1, "unwanted_extensions": test_extensions})
+    def test_has_unwanted_extension_whitelist_mode(self):
+        for filename, result in self.test_params:
+            if filesystem.get_ext(filename):
+                assert filesystem.has_unwanted_extension(filename) is not result
+            else:
+                # missing extension is never considered unwanted
+                assert filesystem.has_unwanted_extension(filename) is False
+
+    @set_config({"unwanted_extensions_mode": 0, "unwanted_extensions": ""})
+    def test_has_unwanted_extension_empty_blacklist(self):
+        for filename, result in self.test_params:
+            assert filesystem.has_unwanted_extension(filename) is False
+
+    @set_config({"unwanted_extensions_mode": 1, "unwanted_extensions": ""})
+    def test_has_unwanted_extension_empty_whitelist(self):
+        for filename, result in self.test_params:
+            if filesystem.get_ext(filename):
+                assert filesystem.has_unwanted_extension(filename) is True
+            else:
+                # missing extension is never considered unwanted
+                assert filesystem.has_unwanted_extension(filename) is False
