@@ -1,5 +1,5 @@
 #!/usr/bin/python3 -OO
-# Copyright 2007-2021 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2022 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -19,55 +19,53 @@
 tests.test_functional_api - Functional tests for the API
 """
 
-import json
-import os
 import shutil
 import stat
-import subprocess
 import sys
-import time
 
 from math import ceil
-from random import choice, randint, sample
+from random import sample
+from typing import List
+
 from tavern.core import run
 from warnings import warn
 
-import sabnzbd.api as api
+import sabnzbd.interface as interface
 from sabnzbd.misc import from_units
 
 from tests.testhelper import *
 
 
 class ApiTestFunctions:
-    """ Collection of (wrapper) functions for API testcases """
+    """Collection of (wrapper) functions for API testcases"""
 
     def _get_api_json(self, mode, extra_args={}):
-        """ Wrapper for API calls with json output """
+        """Wrapper for API calls with json output"""
         extra = {"output": "json", "apikey": SAB_APIKEY}
         extra.update(extra_args)
         return get_api_result(mode=mode, host=SAB_HOST, port=SAB_PORT, extra_arguments=extra)
 
     def _get_api_text(self, mode, extra_args={}):
-        """ Wrapper for API calls with text output """
+        """Wrapper for API calls with text output"""
         extra = {"output": "text", "apikey": SAB_APIKEY}
         extra.update(extra_args)
         return get_api_result(mode=mode, host=SAB_HOST, port=SAB_PORT, extra_arguments=extra)
 
     def _get_api_xml(self, mode, extra_args={}):
-        """ Wrapper for API calls with xml output """
+        """Wrapper for API calls with xml output"""
         extra = {"output": "xml", "apikey": SAB_APIKEY}
         extra.update(extra_args)
         return get_api_result(mode=mode, host=SAB_HOST, port=SAB_PORT, extra_arguments=extra)
 
-    def _setup_script_dir(self, dir, script=None):
+    def _setup_script_dir(self, dir_name, script=None):
         """
         Set the script_dir relative to SAB_CACHE_DIR, copy the example scripts
         there, and add an optional extra script with the given name. To unset
-        the script_dir set the value of dir to an empty string.
+        the script_dir set the value of dir_name to an empty string.
         """
         script_dir_extra = {"section": "misc", "keyword": "script_dir", "value": ""}
-        if dir:
-            script_dir = os.path.join(SAB_CACHE_DIR, dir)
+        if dir_name:
+            script_dir = os.path.join(SAB_CACHE_DIR, dir_name)
             script_dir_extra["value"] = script_dir
             try:
                 if not os.path.exists(script_dir):
@@ -88,14 +86,14 @@ class ApiTestFunctions:
         self._get_api_json("set_config", extra_args=script_dir_extra)
 
     def _record_slots(self, keys):
-        """ Return a list of dicts, storing queue info for the items in iterable 'keys' """
+        """Return a list of dicts, storing queue info for the items in iterable 'keys'"""
         record = []
         for slot in self._get_api_json("queue")["queue"]["slots"]:
             record.append({key: slot[key] for key in keys})
         return record
 
     def _run_tavern(self, test_name, extra_vars=None):
-        """ Run tavern tests in ${test_name}.yaml """
+        """Run tavern tests in ${test_name}.yaml"""
         vars = [
             ("SAB_HOST", SAB_HOST),
             ("SAB_PORT", SAB_PORT),
@@ -115,7 +113,7 @@ class ApiTestFunctions:
         assert result is result.OK
 
     def _get_api_history(self, extra={}):
-        """ Wrapper for history-related api calls """
+        """Wrapper for history-related api calls"""
         # Set a higher default limit; the default is 10 via cfg(history_limit)
         if "limit" not in extra.keys() and "name" not in extra.keys():
             # History calls that use 'name' don't need the limit parameter
@@ -144,9 +142,8 @@ class ApiTestFunctions:
         else:
             minimum_size -= queue_size
 
-        charset = ascii_lowercase + digits
         for _ in range(0, minimum_size):
-            job_name = "%s-CRQ" % ("".join(choice(charset) for i in range(16)))
+            job_name = "%s-CRQ" % random_name()
             job_dir = os.path.join(SAB_CACHE_DIR, job_name)
 
             # Create the job_dir and fill it with a bunch of smallish files with
@@ -154,8 +151,8 @@ class ApiTestFunctions:
             # expect at least two files per NZB.
             try:
                 os.mkdir(job_dir)
-                for number_of_files in range(0, randint(2, 4)):
-                    job_file = "%s.%s" % ("".join(choice(charset) for i in range(randint(6, 18))), sample(charset, 3))
+                for number_of_files in range(0, randint(4, 6)):
+                    job_file = "%s.%s" % (random_name(), random_name(3))
                     with open(os.path.join(job_dir, job_file), "wb") as f:
                         f.write(os.urandom(randint(1, 512 * 1024)))
             except Exception:
@@ -176,21 +173,26 @@ class ApiTestFunctions:
                 warn("Failed to remove %s" % job_dir)
 
     def _purge_queue(self, del_files=0):
-        """ Clear the entire queue """
+        """Clear the entire queue"""
         self._get_api_json("queue", extra_args={"name": "purge", "del_files": del_files})
         assert len(self._get_api_json("queue")["queue"]["slots"]) == 0
 
+    def _get_files(self, nzo_id: str) -> List[str]:
+        files_json = self._get_api_json("get_files", extra_args={"value": nzo_id})
+        assert "files" in files_json
+        return [file["nzf_id"] for file in files_json["files"]]
 
-@pytest.mark.usefixtures("run_sabnzbd_sabnews_and_selenium")
+
+@pytest.mark.usefixtures("run_sabnzbd")
 class TestOtherApi(ApiTestFunctions):
-    """ Test API function not directly involving either history or queue """
+    """Test API function not directly involving either history or queue"""
 
     def test_api_version_testhelper(self):
-        """ Check the version, testhelper style """
+        """Check the version, testhelper style"""
         assert "version" in get_api_result("version", SAB_HOST, SAB_PORT)
 
     def test_api_version_tavern(self):
-        """ Same same, tavern style """
+        """Same same, tavern style"""
         self._run_tavern("api_version")
 
     def test_api_version_json(self):
@@ -203,7 +205,7 @@ class TestOtherApi(ApiTestFunctions):
         assert self._get_api_xml("version")["version"] == sabnzbd.__version__
 
     def test_api_server_stats(self):
-        """ Verify server stats format """
+        """Verify server stats format"""
         self._run_tavern("api_server_stats")
 
     @pytest.mark.parametrize("extra_args", [{}, {"name": "change_complete_action", "value": ""}])
@@ -262,7 +264,7 @@ class TestOtherApi(ApiTestFunctions):
             if round(limit_pct / 100 * linespeed_value) > 20:
                 speed_abs = str(round(limit_pct / 100 * linespeed_value)) + "M"
             else:
-                speed_abs = str(round(limit_pct * 2 ** 10 * linespeed_value / 100)) + "K"
+                speed_abs = str(round(limit_pct * 2**10 * linespeed_value / 100)) + "K"
         else:
             speed_abs = str(round(limit_pct / 100 * from_units(linespeed)))
         assert self._get_api_json("config", extra_args={"name": "speedlimit", "value": speed_abs})["status"] is True
@@ -314,10 +316,11 @@ class TestOtherApi(ApiTestFunctions):
         assert self._get_api_json("set_config_default", extra_args={"keyword": "language"})["status"] is True
 
     def test_api_get_clear_warnings(self):
-        apikey_error = "API Key Incorrect"
         # Trigger warnings by sending requests with a truncated apikey
         for _ in range(0, 2):
-            assert apikey_error in self._get_api_text("shutdown", extra_args={"apikey": SAB_APIKEY[:-1]})
+            assert interface._MSG_APIKEY_INCORRECT in self._get_api_text(
+                "shutdown", extra_args={"apikey": SAB_APIKEY[:-1]}
+            )
 
         # Take delivery of our freshly baked warnings
         json = self._get_api_json("warnings")
@@ -326,7 +329,7 @@ class TestOtherApi(ApiTestFunctions):
         for warning in json["warnings"]:
             for key in ("type", "text", "time"):
                 assert key in warning.keys()
-        assert apikey_error.lower() in json["warnings"][-1]["text"].lower()
+        assert interface._MSG_APIKEY_INCORRECT.lower() in json["warnings"][-1]["text"].lower()
 
         # Clear all warnings
         assert self._get_api_json("warnings", extra_args={"name": "clear"})["status"] is True
@@ -404,12 +407,12 @@ class TestOtherApi(ApiTestFunctions):
             )
 
 
-@pytest.mark.usefixtures("run_sabnzbd_sabnews_and_selenium")
+@pytest.mark.usefixtures("run_sabnzbd")
 class TestQueueApi(ApiTestFunctions):
-    """ Test queue-related API responses """
+    """Test queue-related API responses"""
 
     def test_api_queue_empty_format(self):
-        """ Verify formatting, presence of fields for empty queue """
+        """Verify formatting, presence of fields for empty queue"""
         self._purge_queue()
         self._run_tavern("api_queue_empty")
 
@@ -493,7 +496,7 @@ class TestQueueApi(ApiTestFunctions):
 
     @pytest.mark.parametrize("select_by", ["search", "nzo_ids"])
     def test_api_queue_restrict_no_results_search_and_nzo_ids(self, select_by):
-        fake_search = "FakeSearch_%s" % ("".join(choice(ascii_lowercase + digits) for i in range(16)))
+        fake_search = "FakeSearch_%s" % random_name()
         json = self._get_api_json("queue", extra_args={select_by: fake_search})
         assert len(json["queue"]["slots"]) == 0
 
@@ -545,10 +548,10 @@ class TestQueueApi(ApiTestFunctions):
         self._create_random_queue(minimum_size=1)
 
         # Setup the script_dir as ordered
-        dir = ""
+        script_dir = ""
         if set_scriptsdir:
-            dir = "scripts"
-        self._setup_script_dir(dir, script="my_script_for_sab.py")
+            script_dir = "scripts"
+        self._setup_script_dir(script_dir, script="my_script_for_sab.py")
 
         # Run the queue complete action api call
         prev_value = self._get_api_json("queue")["queue"]["finishaction"]
@@ -569,7 +572,7 @@ class TestQueueApi(ApiTestFunctions):
         self._get_api_json("queue", extra_args={"name": "change_complete_action", "value": ""})
 
     def test_api_queue_single_format(self):
-        """ Verify formatting, presence of fields for single queue entry """
+        """Verify formatting, presence of fields for single queue entry"""
         self._create_random_queue(minimum_size=1)
         self._run_tavern("api_queue_format")
 
@@ -612,7 +615,7 @@ class TestQueueApi(ApiTestFunctions):
         def size_in_bytes(size):
             # Helper function for list.sort() to deal with B/KB/MB in size values
             if size.endswith(" MB"):
-                return float(size.strip(" MB")) * 1024 ** 2
+                return float(size.strip(" MB")) * 1024**2
             if size.endswith(" KB"):
                 return float(size.strip(" KB")) * 1024
             if size.endswith(" B"):
@@ -848,7 +851,7 @@ class TestQueueApi(ApiTestFunctions):
                 assert changed[row] == original[row]
 
     def test_api_queue_get_files_format(self):
-        """ Verify formatting, presence of fields for mode=get_files """
+        """Verify formatting, presence of fields for mode=get_files"""
         self._create_random_queue(minimum_size=1)
         nzo_id = self._get_api_json("queue")["queue"]["slots"][0]["nzo_id"]
         # Pass the nzo_id this way rather than fetching it in a tavern stage, as
@@ -862,9 +865,7 @@ class TestQueueApi(ApiTestFunctions):
         # Select a job and file to delete
         nzo_ids = [slot["nzo_id"] for slot in self._get_api_json("queue")["queue"]["slots"]]
         nzo_id = choice(nzo_ids)
-        json = self._get_api_json("get_files", extra_args={"value": nzo_id})
-        assert json["files"]
-        nzf_ids = [file["nzf_id"] for file in json["files"]]
+        nzf_ids = self._get_files(nzo_id)
         assert nzf_ids
         nzf_id = choice(nzf_ids)
 
@@ -874,8 +875,7 @@ class TestQueueApi(ApiTestFunctions):
         assert nzf_id in json["nzf_ids"]
 
         # Verify it's really gone
-        json = self._get_api_json("get_files", extra_args={"value": nzo_id})
-        changed_nzf_ids = [file["nzf_id"] for file in json["files"]]
+        changed_nzf_ids = self._get_files(nzo_id)
         assert nzf_id not in changed_nzf_ids
         assert len(changed_nzf_ids) == len(nzf_ids) - 1
 
@@ -884,25 +884,85 @@ class TestQueueApi(ApiTestFunctions):
         assert json["status"] is False
         assert json["nzf_ids"] == []
 
-        # Attempt to remove multiple nzf_ids at once (which isn't supported)
+        # Attempt to remove multiple nzf_ids at once
         nzo_ids.remove(nzo_id)
         nzo_id = choice(nzo_ids)
-        json = self._get_api_json("get_files", extra_args={"value": nzo_id})
-        nzf_ids = [file["nzf_id"] for file in json["files"]]
-        assert len(nzf_ids) > 0
+        original_nzf_ids = self._get_files(nzo_id)
+        nzf_ids_to_remove = sample(original_nzf_ids, k=2)
         json = self._get_api_json(
-            mode="queue", extra_args={"name": "delete_nzf", "value": nzo_id, "value2": ",".join(nzf_ids)}
+            mode="queue", extra_args={"name": "delete_nzf", "value": nzo_id, "value2": ",".join(nzf_ids_to_remove)}
+        )
+        assert json["status"] is True
+        assert len(json["nzf_ids"]) == len(nzf_ids_to_remove)
+
+        # Verify they are really gone
+        changed_nzf_ids = self._get_files(nzo_id)
+        for nzf_id in nzf_ids_to_remove:
+            assert nzf_id not in changed_nzf_ids
+            assert len(changed_nzf_ids) == len(original_nzf_ids) - len(nzf_ids_to_remove)
+
+    def test_api_move_nzf_bulk(self):
+        # Clear queue so we don't use any of the files from the
+        # delete_nzf test, since we expect at least 4 files
+        self._purge_queue()
+        self._create_random_queue(minimum_size=1)
+
+        # Select a job and file to delete
+        nzo_ids = [slot["nzo_id"] for slot in self._get_api_json("queue")["queue"]["slots"]]
+        nzo_id = choice(nzo_ids)
+        nzf_ids = self._get_files(nzo_id)
+        assert nzf_ids
+
+        # Move the bottom 2 up
+        json = self._get_api_json(
+            mode="move_nzf_bulk", extra_args={"name": "top", "value": nzo_id, "nzf_ids": ",".join(nzf_ids[-2:])}
+        )
+        assert json["status"] is True
+        new_nzf_ids = self._get_files(nzo_id)
+        assert new_nzf_ids[:2] == nzf_ids[-2:]
+
+        # Move the top 2 down
+        nzf_ids = self._get_files(nzo_id)
+        json = self._get_api_json(
+            mode="move_nzf_bulk", extra_args={"name": "bottom", "value": nzo_id, "nzf_ids": ",".join(nzf_ids[:2])}
+        )
+        assert json["status"] is True
+        new_nzf_ids = self._get_files(nzo_id)
+        assert new_nzf_ids[-2:] == nzf_ids[:2]
+
+        # Move the last 2 up 2 spots
+        nzf_ids = self._get_files(nzo_id)
+        json = self._get_api_json(
+            mode="move_nzf_bulk",
+            extra_args={"name": "up", "value": nzo_id, "nzf_ids": ",".join(nzf_ids[-2:]), "size": "2"},
+        )
+        assert json["status"] is True
+        new_nzf_ids = self._get_files(nzo_id)
+        assert new_nzf_ids[-4:-2] == nzf_ids[-2:]
+
+        # Move the first 2 down 2 spots
+        nzf_ids = self._get_files(nzo_id)
+        json = self._get_api_json(
+            mode="move_nzf_bulk",
+            extra_args={"name": "down", "value": nzo_id, "nzf_ids": ",".join(nzf_ids[:2]), "size": "2"},
+        )
+        assert json["status"] is True
+        new_nzf_ids = self._get_files(nzo_id)
+        assert new_nzf_ids[2:4] == nzf_ids[:2]
+
+        # See failure if called with valid nzf_id but missing size
+        json = self._get_api_json(
+            mode="move_nzf_bulk", extra_args={"name": "up", "value": nzo_id, "nzf_ids": ",".join(nzf_ids[:2])}
         )
         assert json["status"] is False
-        assert json["nzf_ids"] == []
 
 
-@pytest.mark.usefixtures("run_sabnzbd_sabnews_and_selenium", "generate_fake_history", "update_history_specs")
+@pytest.mark.usefixtures("run_sabnzbd", "generate_fake_history", "update_history_specs")
 class TestHistoryApi(ApiTestFunctions):
-    """ Test history-related API responses """
+    """Test history-related API responses"""
 
     def test_api_history_format(self):
-        """ Verify formatting, presence of expected history fields """
+        """Verify formatting, presence of expected history fields"""
         # Checks all output styles: json, text and xml
         self._run_tavern("api_history_format")
 
@@ -929,7 +989,7 @@ class TestHistoryApi(ApiTestFunctions):
         assert slot_sum == slot_total
 
     def test_api_history_restrict_invalid_cat(self):
-        fake_cat = "FakeCat_%s" % ("".join(choice(ascii_lowercase + digits) for i in range(16)))
+        fake_cat = "FakeCat_%s" % random_name()
         json = self._get_api_history({"category": fake_cat})
         assert len(json["history"]["slots"]) == 0
 
@@ -944,12 +1004,12 @@ class TestHistoryApi(ApiTestFunctions):
                 assert slot["category"] == cat
 
     def test_api_history_restrict_invalid_cat_and_limit(self):
-        fake_cat = "FakeCat_%s" % ("".join(choice(ascii_lowercase + digits) for i in range(16)))
+        fake_cat = "FakeCat_%s" % random_name()
         json = self._get_api_history({"category": fake_cat, "limit": randint(1, 5)})
         assert len(json["history"]["slots"]) == 0
 
     def test_api_history_restrict_invalid_cat_and_search(self):
-        fake_cat = "FakeCat_%s" % ("".join(choice(ascii_lowercase + digits) for i in range(16)))
+        fake_cat = "FakeCat_%s" % random_name()
         for distro in self.history_distro_names:
             json = self._get_api_history({"category": fake_cat, "search": distro})
             assert len(json["history"]["slots"]) == 0
@@ -972,12 +1032,12 @@ class TestHistoryApi(ApiTestFunctions):
 
     @pytest.mark.parametrize("select_by", ["search", "nzo_ids"])
     def test_api_history_restrict_no_results_search_and_nzo_ids(self, select_by):
-        fake_search = "FakeSearch_%s" % ("".join(choice(ascii_lowercase + digits) for i in range(16)))
+        fake_search = "FakeSearch_%s" % random_name()
         json = self._get_api_history({select_by: fake_search})
         assert len(json["history"]["slots"]) == 0
 
     def test_api_history_restrict_cat_and_search_and_limit(self):
-        """ Combine search, category and limits requirements into a single query """
+        """Combine search, category and limits requirements into a single query"""
         limit_sum = 0
         slot_sum = 0
         limits = [randint(1, ceil(self.history_size / 10)) for _ in range(0, len(self.history_distro_names))]
@@ -1087,7 +1147,7 @@ class TestHistoryApi(ApiTestFunctions):
             assert slot["status"] != Status.COMPLETED
 
 
-@pytest.mark.usefixtures("run_sabnzbd_sabnews_and_selenium", "generate_fake_history", "update_history_specs")
+@pytest.mark.usefixtures("run_sabnzbd", "generate_fake_history", "update_history_specs")
 class TestHistoryApiPart2(ApiTestFunctions):
     """Test history-related API responses, part 2. A separate testcase is
     needed because the previous one ran out of history entries to delete."""
@@ -1114,6 +1174,6 @@ class TestHistoryApiPart2(ApiTestFunctions):
         assert json["history"]["noofslots"] == 0
 
     def test_api_history_empty_format(self):
-        """ Verify formatting, presence of fields for empty history """
+        """Verify formatting, presence of fields for empty history"""
         # Checks all output styles: json, text and xml
         self._run_tavern("api_history_empty")
