@@ -1,5 +1,5 @@
 #!/usr/bin/python3 -OO
-# Copyright 2007-2020 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2022 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -18,19 +18,22 @@
 """
 tests.test_functional_misc - Functional tests of various functions
 """
+import io
 import shutil
 import subprocess
 import sys
 
 import sabnzbd.encoding
+from sabnzbd.filesystem import save_compressed
+from sabnzbd.constants import JOB_ADMIN
 from tests.testhelper import *
 
 
 class TestShowLogging(SABnzbdBaseTest):
     def test_showlog(self):
-        """ Test the output of the filtered-log button """
+        """Test the output of the filtered-log button"""
         # Basic URL-fetching, easier than Selenium file download
-        log_result = get_url_result("status/showlog")
+        log_result = get_api_result("showlog", extra_arguments={"output": "text"})
 
         # Make sure it has basic log stuff
         assert "The log" in log_result
@@ -41,9 +44,56 @@ class TestShowLogging(SABnzbdBaseTest):
         assert "[misc]" in log_result
 
 
+class TestQueueRepair(SABnzbdBaseTest):
+    def test_queue_repair(self):
+        """Test full queue repair by manually adding an orphaned job"""
+        nzb_fp = create_and_read_nzb_fp("basic_rar5")
+        test_job_name = "testfile_%s" % time.time()
+
+        # Create folder and save compressed NZB like SABnzbd would do
+        admin_path = os.path.join(SAB_INCOMPLETE_DIR, test_job_name, JOB_ADMIN)
+        os.makedirs(admin_path)
+        save_compressed(admin_path, test_job_name, nzb_fp)
+        assert os.path.exists(os.path.join(admin_path, test_job_name + ".nzb.gz"))
+
+        # Pause the queue do we don't download stuff
+        assert get_api_result("pause") == {"status": True}
+
+        # Request queue repair
+        assert get_api_result("restart_repair") == {"status": True}
+
+        # Let's check the queue, this can take long on GitHub Actions
+        for _ in range(60):
+            queue_result_slots = {}
+            try:
+                # Can give timeout if still restarting
+                queue_result_slots = get_api_result("queue", extra_arguments={"limit": 10000})["queue"]["slots"]
+            except requests.exceptions.RequestException:
+                pass
+
+            # Check if the repaired job was added to the queue
+            if queue_result_slots:
+                break
+            time.sleep(1)
+        else:
+            # The loop never stopped, so we fail
+            pytest.fail("Did not find the repaired job in the queue")
+            return
+
+        # Verify filename
+        assert test_job_name in [slot["filename"] for slot in queue_result_slots]
+
+        # Let's remove this thing
+        get_api_result("queue", extra_arguments={"name": "delete", "value": "all"})
+        assert len(get_api_result("queue")["queue"]["slots"]) == 0
+
+        # Unpause
+        assert get_api_result("resume") == {"status": True}
+
+
 class TestSamplePostProc:
     def test_sample_post_proc(self):
-        """ Make sure we don't break things """
+        """Make sure we don't break things"""
         # Set parameters
         script_params = [
             "somedir222",
@@ -78,7 +128,7 @@ class TestSamplePostProc:
 
 class TestExtractPot:
     def test_extract_pot(self):
-        """ Simple test if translation extraction still works """
+        """Simple test if translation extraction still works"""
         script_call = [sys.executable, "tools/extract_pot.py"]
 
         # Run script and check output
@@ -99,6 +149,15 @@ class TestExtractPot:
         assert (cur_time - os.path.getmtime("po/main/SABnzbd.pot")) < 30
         assert (cur_time - os.path.getmtime("po/email/SABemail.pot")) < 30
 
+        # Reset the translation updates
+        try:
+            lang_command = "git checkout @ -- %s/../po/main/SABnzbd.pot" % SAB_BASE_DIR
+            subprocess.Popen(lang_command.split()).communicate(timeout=30)
+            lang_command = "git checkout @ -- %s/../po/email/SABemail.pot" % SAB_BASE_DIR
+            subprocess.Popen(lang_command.split()).communicate(timeout=30)
+        except:
+            pass
+
 
 @pytest.mark.skipif(sys.platform.startswith("win"), reason="Skipping on Windows")
 @pytest.mark.skipif(sys.platform.startswith("darwin"), reason="Fails for now due to PyObjC problem")
@@ -156,6 +215,10 @@ class TestDaemonizing(SABnzbdBaseTest):
         assert os.path.exists(error_log_path)
         assert os.path.getsize(error_log_path) < 1024
 
-        # Let's shut it down and give it some time to do so
-        get_url_result("shutdown", daemon_host, daemon_port)
-        time.sleep(3.0)
+        try:
+            # Let's shut it down and give it some time to do so
+            get_url_result("shutdown", daemon_host, daemon_port)
+            time.sleep(3.0)
+        except requests.exceptions.RequestException:
+            # Shutdown can be faster than the request
+            pass
