@@ -1,5 +1,5 @@
 #!/usr/bin/python3 -OO
-# Copyright 2007-2021 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2022 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -124,12 +124,12 @@ class TestMisc:
         assert "10.0 M" == misc.to_units(1024 * 1024 * 10)
         assert "100.0 M" == misc.to_units(1024 * 1024 * 100)
         assert "9.8 G" == misc.to_units(1024 * 1024 * 10000)
-        assert "1024.0 P" == misc.to_units(1024 ** 6)
+        assert "1024.0 P" == misc.to_units(1024**6)
 
     def test_unit_back_and_forth(self):
         assert 100 == misc.from_units(misc.to_units(100))
         assert 1024 == misc.from_units(misc.to_units(1024))
-        assert 1024 ** 3 == misc.from_units(misc.to_units(1024 ** 3))
+        assert 1024**3 == misc.from_units(misc.to_units(1024**3))
 
     def test_caller_name(self):
         @set_config({"log_level": 0})
@@ -193,6 +193,27 @@ class TestMisc:
         assert "1 Stunde 1 Minuten 1 Sekunde" == misc.format_time_string(60 * 60 + 60 + 1)
         assert "1 Tag 59 Sekunden" == misc.format_time_string(86400 + 59)
         assert "2 Tage 2 Stunden 2 Sekunden" == misc.format_time_string(2 * 86400 + 2 * 60 * 60 + 2)
+        # Reset language
+        lang.set_language()
+
+    def test_format_time_left(self):
+        assert "0:00:00" == misc.format_time_left(0)
+        assert "0:00:00" == misc.format_time_left(-1)
+        assert "0:00:01" == misc.format_time_left(1)
+        assert "0:01:01" == misc.format_time_left(60 + 1)
+        assert "0:11:10" == misc.format_time_left(60 * 11 + 10)
+        assert "3:11:10" == misc.format_time_left(60 * 60 * 3 + 60 * 11 + 10)
+        assert "13:11:10" == misc.format_time_left(60 * 60 * 13 + 60 * 11 + 10)
+        assert "1:09:11:10" == misc.format_time_left(60 * 60 * 33 + 60 * 11 + 10)
+
+    def test_format_time_left_short(self):
+        assert "0:00" == misc.format_time_left(0, short_format=True)
+        assert "0:01" == misc.format_time_left(1, short_format=True)
+        assert "1:01" == misc.format_time_left(60 + 1, short_format=True)
+        assert "11:10" == misc.format_time_left(60 * 11 + 10, short_format=True)
+        assert "3:11:10" == misc.format_time_left(60 * 60 * 3 + 60 * 11 + 10, short_format=True)
+        assert "13:11:10" == misc.format_time_left(60 * 60 * 13 + 60 * 11 + 10, short_format=True)
+        assert "1:09:11:10" == misc.format_time_left(60 * 60 * 33 + 60 * 11 + 10, short_format=True)
 
     def test_int_conv(self):
         assert 0 == misc.int_conv("0")
@@ -214,6 +235,44 @@ class TestMisc:
         os.unlink("test.cert")
         os.unlink("test.key")
 
+    @pytest.mark.parametrize(
+        "name, result",
+        [
+            ("Free.Open.Source.Movie.2001.1080p.WEB-DL.DD5.1.H264-FOSS", False),  # Not samples
+            ("Setup.exe", False),
+            ("23.123.hdtv-rofl", False),
+            ("Something.1080p.WEB-DL.DD5.1.H264-EMRG-sample", True),  # Samples
+            ("Something.1080p.WEB-DL.DD5.1.H264-EMRG-sample.ogg", True),
+            ("Sumtin_Else_1080p_WEB-DL_DD5.1_H264_proof-EMRG", True),
+            ("Wot.Eva.540i.WEB-DL.aac.H264-Groupie sample.mp4", True),
+            ("file-sample.mkv", True),
+            ("PROOF.JPG", True),
+            ("Bla.s01e02.title.1080p.aac-sample proof.mkv", True),
+            ("Bla.s01e02.title.1080p.aac-proof.mkv", True),
+            ("Bla.s01e02.title.1080p.aac sample proof.mkv", True),
+            ("Bla.s01e02.title.1080p.aac proof.mkv", True),
+            ("Lwtn.s08e26.1080p.web.h264-glhf-sample.par2", True),
+            ("Lwtn.s08e26.1080p.web.h264-glhf-sample.vol001-002.par2", True),
+            ("Look at That 2011 540i WEB-DL.H265-NoSample", False),
+        ],
+    )
+    def test_is_sample(self, name, result):
+        assert misc.is_sample(name) == result
+
+    @pytest.mark.parametrize(
+        "name, result",
+        [
+            ("Not Death Proof (2022) 1080p x264 (DD5.1) BE Subs", False),  # Try to trigger some false positives
+            ("Proof.of.Everything.(2042).4320p.x266-4U", False),
+            ("Crime_Scene_S01E13_Free_Sample_For_Sale_480p-OhDear", False),
+            ("Sample That 2011 480p WEB-DL.H265-aMiGo", False),
+            ("NOT A SAMPLE.JPG", False),
+        ],
+    )
+    def test_is_sample_known_false_positives(self, name, result):
+        """We know these fail, but don't have a better solution for them at the moment."""
+        assert misc.is_sample(name) != result
+
     @pytest.mark.parametrize(
         "test_input, expected_output",
         [
@@ -224,12 +283,27 @@ class TestMisc:
         ],
     )
     def test_list_to_cmd(self, test_input, expected_output):
-        """ Test to convert list to a cmd.exe-compatible command string """
-
+        """Test to convert list to a cmd.exe-compatible command string"""
         res = misc.list2cmdline(test_input)
         # Make sure the output is cmd.exe-compatible
         assert res == expected_output
 
+    def test_recursive_html_escape(self):
+        """Very basic test if the recursive clean-up works"""
+        input_test = {
+            "foo": "<b>?ar'\"",
+            "test_list": ["test&1", 'test"2'],
+            "test_nested_list": [["test&1", 'test"2', 4]],
+            "test_dict": {"test": ["test<>1", "#"]},
+        }
+        # Dict is updated in-place
+        misc.recursive_html_escape(input_test)
+        # Have to check them by hand
+        assert input_test["foo"] == "&lt;b&gt;?ar&#x27;&quot;"
+        assert input_test["test_list"] == ["test&amp;1", "test&quot;2"]
+        assert input_test["test_nested_list"] == [["test&amp;1", "test&quot;2", 4]]
+        assert input_test["test_dict"]["test"] == ["test&lt;&gt;1", "#"]
+
     @pytest.mark.parametrize(
         "value, result",
         [
@@ -248,6 +322,7 @@ class TestMisc:
             ("", False),
             ("3.2.0", False),
             (-42, False),
+            ("::ffff:192.168.1.100", False),
         ],
     )
     def test_is_ipv4_addr(self, value, result):
@@ -276,6 +351,7 @@ class TestMisc:
             ("[1.2.3.4]", False),
             ("2001:1", False),
             ("2001::[2001::1]", False),
+            ("::ffff:192.168.1.100", True),
         ],
     )
     def test_is_ipv6_addr(self, value, result):
@@ -305,6 +381,9 @@ class TestMisc:
             ("[127.6.6.6]", False),
             ("2001:1", False),
             ("2001::[2001::1]", False),
+            ("::ffff:192.168.1.100", False),
+            ("::ffff:1.1.1.1", False),
+            ("::ffff:127.0.0.1", True),
         ],
     )
     def test_is_loopback_addr(self, value, result):
@@ -337,6 +416,9 @@ class TestMisc:
             ("[127.6.6.6]", False),
             ("2001:1", False),
             ("2001::[2001::1]", False),
+            ("::ffff:192.168.1.100", False),
+            ("::ffff:1.1.1.1", False),
+            ("::ffff:127.0.0.1", True),
         ],
     )
     def test_is_localhost(self, value, result):
@@ -373,11 +455,97 @@ class TestMisc:
             ("[1.2.3.4]", False),
             ("2001:1", False),
             ("2001::[2001::1]", False),
+            ("::ffff:192.168.1.100", True),
+            ("::ffff:1.1.1.1", False),
+            ("::ffff:127.0.0.1", False),
         ],
     )
     def test_is_lan_addr(self, value, result):
         assert misc.is_lan_addr(value) is result
 
+    @pytest.mark.parametrize(
+        "ip, subnet, result",
+        [
+            ("2001:c0f:fee::1", "2001:c0f:fee", True),  # Old-style range setting
+            ("2001:c0f:fee::1", "2001:c0f:FEE:", True),
+            ("2001:c0f:fee::1", "2001:c0FF:ffee", False),
+            ("2001:c0f:fee::1", "2001:c0ff:ffee:", False),
+            ("2001:C0F:FEE::1", "2001:c0f:fee::/48", True),
+            ("2001:c0f:fee::1", "2001:c0f:fee::/112", True),
+            ("2001:c0f:fee::1", "::/0", True),  # Subnet equals the entire IPv6 address space
+            ("2001:c0f:fee::1", "2001:c0:ffee::/48", False),
+            ("2001:c0f:fee::1", "2001:c0ff:ee::/112", False),
+            ("2001:c0f:fEE::1", "2001:c0f:fee:eeee::/48", False),  # Invalid subnet
+            ("2001:c0f:Fee::1", "2001:c0f:fee:/64", False),
+            ("2001:c0f:fee::1", "2001:c0f:fee:eeee:3:2:1:0/112", False),
+            ("2001:c0f:fee::1", "2001:c0f:fee::1", True),  # Single-IP subnet
+            ("2001:c0f:fee::1", "2001:c0f:fee::1/128", True),
+            ("2001:c0f:fee::1", "2001:c0f:fee::2", False),
+            ("2001:c0f:fee::1", "2001:c0f:fee::2/128", False),
+            ("::1", "::/127", True),
+            ("::1", "2021::/64", False),  # Localhost not in subnet
+            ("192.168.43.21", "192.168.43", True),  # Old-style subnet setting
+            ("192.168.43.21", "192.168.43.", True),
+            ("192.168.43.21", "192.168.4", False),
+            ("192.168.43.21", "192.168.4.", False),
+            ("10.11.12.13", "10", True),  # Bad old-style setting (allowed 100.0.0.0/6, 104.0.0.0/6 and 108.0.0.0/7)
+            ("10.11.12.13", "10.", True),  # Correct version of the same (10.0.0.0/8 only)
+            ("108.1.2.3", "10", False),  # This used to be allowed with the bad setting!
+            ("108.1.2.3", "10.", False),
+            ("192.168.43.21", "192.168.0.0/16", True),
+            ("192.168.43.21", "192.168.0.0/255.255.255.0", True),
+            ("::ffff:192.168.43.21", "192.168.43.0/24", True),  # IPv4-mapped IPv6 ("dual-stack") notation
+            ("::FFff:192.168.43.21", "192.168.43.0/24", True),
+            ("::ffff:192.168.12.34", "192.168.43.0/24", False),
+            ("::ffFF:192.168.12.34", "192.168.43.0/24", False),
+            ("192.168.43.21", "192.168.43.0/26", True),
+            ("200.100.50.25", "0.0.0.0/0", True),  # Subnet equals the entire IPv4 address space
+            ("192.168.43.21", "10.0.0.0/8", False),
+            ("192.168.43.21", "192.168.1.0/22", False),
+            ("192.168.43.21", "192.168.43.21/24", False),  # Invalid subnet
+            ("192.168.43.21", "192.168.43/24", False),
+            ("192.168.43.21", "192.168.43.0/16", False),
+            ("192.168.43.21", "192.168.43.0/255.252.0.0", False),
+            ("192.168.43.21", "192.168.43.21", True),  # Single-IP subnet
+            ("192.168.43.21", "192.168.43.21/32", True),
+            ("192.168.43.21", "192.168.43.21/255.255.255.255", True),
+            ("192.168.43.21", "192.168.43.12", False),
+            ("192.168.43.21", "192.168.43.0/32", False),
+            ("192.168.43.21", "43.21.168.192/255.255.255.255", False),
+            ("127.0.0.1", "127.0.0.0/31", True),
+            ("127.0.1.1", "127.0.0.0/24", False),  # Localhost not in subnet
+            ("111.222.33.44", "111:222:33::/96", False),  # IPv4/IPv6 mixup
+            ("111:222:33::44", "111.222.0.0/24", False),
+            ("aaaa::1:2:3:4", "f:g:h:i:43:21::/112", False),  # Invalid subnet
+            ("4.3.2.1", "654.3.2.1.0/24", False),
+            (None, "1.2.3.4/32", False),  # Missing input
+            ("1:a:2:b::", None, False),
+            (None, None, False),
+        ],
+    )
+    def test_ip_in_subnet(self, ip, subnet, result):
+        misc.ip_in_subnet(ip, subnet) is result
+
+    @pytest.mark.parametrize(
+        "ip, result",
+        [
+            ("::ffff:127.0.0.1", "127.0.0.1"),
+            ("::FFFF:127.0.0.1", "127.0.0.1"),
+            ("::ffff:192.168.1.255", "192.168.1.255"),
+            ("::ffff:8.8.8.8", "8.8.8.8"),
+            ("2007::2021", "2007::2021"),
+            ("::ffff:2007:2021", "::ffff:2007:2021"),
+            ("2007::ffff:2021", "2007::ffff:2021"),
+            ("12.34.56.78", "12.34.56.78"),
+            ("foobar", "foobar"),
+            ("0:0:0:0:0:ffff:8.8.4.4", "8.8.4.4"),
+            ("0000:0000:0000:0000:0000:ffff:1.0.0.1", "1.0.0.1"),
+            ("0000::0:ffff:1.1.1.1", "1.1.1.1"),
+        ],
+    )
+    def test_strip_ipv4_mapped_notation(self, ip, result):
+        misc.strip_ipv4_mapped_notation(ip) == result
+
 
 class TestBuildAndRunCommand:
     # Path should exist
@@ -434,7 +602,11 @@ class TestBuildAndRunCommand:
         temp_file_fd, temp_file_path = tempfile.mkstemp(suffix=".py")
         os.close(temp_file_fd)
         misc.build_and_run_command([temp_file_path, "input 1"])
-        assert mock_subproc_popen.call_args[0][0] == ["python", temp_file_path, "input 1"]
+        assert mock_subproc_popen.call_args[0][0] == [
+            sys.executable if sys.executable else "python",
+            temp_file_path,
+            "input 1",
+        ]
         os.remove(temp_file_path)
 
         # Have to fake these for it to work
