@@ -1,9 +1,12 @@
 """
 tests.test_nzbstuff - Testing functions in nzbstuff.py
 """
+import io
+
 import sabnzbd.nzbstuff as nzbstuff
 from sabnzbd.config import ConfigCat
 from sabnzbd.constants import NORMAL_PRIORITY
+from sabnzbd.encoding import utob
 from sabnzbd.filesystem import globber
 
 from tests.testhelper import *
@@ -21,13 +24,12 @@ class TestNZO:
         nzo = nzbstuff.NzbObject("test_basic")
         assert nzo.work_name == "test_basic"
         assert not nzo.files
-        assert not nzo.created
 
         # Create NZB-file to import
-        nzb_data = create_and_read_nzb("basic_rar5")
+        nzb_fp = create_and_read_nzb_fp("basic_rar5")
 
         # Very basic test of NZO creation with data
-        nzo = nzbstuff.NzbObject("test_basic_data", nzb=nzb_data)
+        nzo = nzbstuff.NzbObject("test_basic_data", nzb_fp=nzb_fp)
         assert nzo.final_name == "test_basic_data"
         assert nzo.files
         assert nzo.files[0].filename == "testfile.rar"
@@ -122,17 +124,26 @@ class TestNZBStuffHelpers:
                 "REQ Author Child's The Book-Thanks much - Child, Lee - Author - The Book.epub",
             ),
             ('63258-0[001/101] - "63258-2.0" yEnc (1/250) (1/250)', "63258-2.0"),
+            # If specified between ", the extension is allowed to be too long
+            ('63258-0[001/101] - "63258-2.0toolong" yEnc (1/250) (1/250)', "63258-2.0toolong"),
             (
                 "Singer - A Album (2005) - [04/25] - 02 Sweetest Somebody (I Know).flac",
-                "- 02 Sweetest Somebody (I Know).flac",
+                "Singer - A Album (2005) - [04/25] - 02 Sweetest Somebody (I Know).flac",
             ),
             ("<>random!>", "<>random!>"),
             ("nZb]-[Supertje-_S03E11-12_", "nZb]-[Supertje-_S03E11-12_"),
             ("Bla [Now it's done.exe]", "Now it's done.exe"),
+            # If specified between [], the extension should be a valid one
+            ("Bla [Now it's done.123nonsense]", "Bla [Now it's done.123nonsense]"),
             (
                 '[PRiVATE]-[WtFnZb]-[Video_(2001)_AC5.1_-RELEASE_[TAoE].mkv]-[1/23] - "" yEnc 1234567890 (1/23456)',
                 '[PRiVATE]-[WtFnZb]-[Video_(2001)_AC5.1_-RELEASE_[TAoE].mkv]-[1/23] - "" yEnc 1234567890 (1/23456)',
             ),
+            (
+                "[PRiVATE]-[WtFnZb]-[219]-[1/serie.name.s01e01.1080p.web.h264-group.mkv] - "
+                " yEnc (1/[PRiVATE] \\c2b510b594\\::686ea969999193.155368eba4965e56a8cd263382e012.f2712fdc::/97bd201cf931/) 1 (1/0)",
+                "serie.name.s01e01.1080p.web.h264-group.mkv",
+            ),
         ],
     )
     def test_name_extractor(self, subject, filename):
