@@ -9,7 +9,6 @@ tests.test_postproc- Tests of various functions in newspack, among which rar_ren
 """
 
 import shutil
-from distutils.dir_util import copy_tree
 from unittest import mock
 
 from sabnzbd.postproc import *
@@ -35,15 +34,15 @@ class TestPostProc:
 
             # create a fresh copy
             try:
-                # shutil.copytree(sourcedir, workingdir) gives problems on AppVeyor, so:
-                copy_tree(sourcedir, workingdir)
+                shutil.copytree(sourcedir, workingdir)
             except:
                 pytest.fail("Could not create copy of files for rar_renamer")
 
             # And now let the magic happen:
             nzo = mock.Mock()
             nzo.final_name = "somedownloadname"
-            number_renamed_files = rar_renamer(nzo, workingdir)
+            nzo.download_path = workingdir
+            number_renamed_files = rar_renamer(nzo)
 
             # run check on the resulting files
             if expected_filename_matches:
@@ -74,3 +73,19 @@ class TestPostProc:
         sourcedir = os.path.join(SAB_DATA_DIR, "obfuscated_but_no_rar")
         expected_filename_matches = {"*.rar": 0, "*-*-*-*-*": 6}
         assert deobfuscate_dir(sourcedir, expected_filename_matches) == 0
+
+        # One obfuscated rar set, but first rar (.part1.rar) is missing
+        sourcedir = os.path.join(SAB_DATA_DIR, "obfuscated_single_rar_set_missing_first_rar")
+        # single rar set (of 6 obfuscated rar files), so we expect renaming
+        # thus result must 6 rar files, and 0 obfuscated files
+        expected_filename_matches = {"*.rar": 6, "*-*-*-*-*": 0}
+        # 6 files should have been renamed
+        assert deobfuscate_dir(sourcedir, expected_filename_matches) == 6
+
+        # Two obfuscated rar sets, but some rars are missing
+        sourcedir = os.path.join(SAB_DATA_DIR, "obfuscated_double_rar_set_missing_rar")
+        # Two sets, missing rar, so we expect no renaming
+        # thus result should be 0 rar files, and still 8 obfuscated files
+        expected_filename_matches = {"*.rar": 0, "*-*-*-*-*": 8}
+        # 0 files should have been renamed
+        assert deobfuscate_dir(sourcedir, expected_filename_matches) == 0
