@@ -1,5 +1,5 @@
 #!/usr/bin/python3 -OO
-# Copyright 2007-2021 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2022 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -18,10 +18,12 @@
 """
 tests.testhelper - Basic helper functions
 """
-
+import io
 import os
 import time
 from http.client import RemoteDisconnected
+from typing import BinaryIO, Optional, Dict
+
 import pytest
 from random import choice, randint
 import requests
@@ -65,7 +67,7 @@ SAB_NEWSSERVER_PORT = 8888
 
 
 def set_config(settings_dict):
-    """ Change config-values on the fly, per test"""
+    """Change config-values on the fly, per test"""
 
     def set_config_decorator(func):
         def wrapper_func(*args, **kwargs):
@@ -77,8 +79,8 @@ def set_config(settings_dict):
             value = func(*args, **kwargs)
 
             # Reset values
-            for item, val in settings_dict.items():
-                getattr(cfg, item).default()
+            for item in settings_dict:
+                getattr(cfg, item).set(getattr(cfg, item).default())
             return value
 
         return wrapper_func
@@ -87,7 +89,7 @@ def set_config(settings_dict):
 
 
 def set_platform(platform):
-    """ Change config-values on the fly, per test"""
+    """Change config-values on the fly, per test"""
 
     def set_platform_decorator(func):
         def wrapper_func(*args, **kwargs):
@@ -121,13 +123,13 @@ def set_platform(platform):
 
 
 def get_url_result(url="", host=SAB_HOST, port=SAB_PORT):
-    """ Do basic request to web page """
+    """Do basic request to web page"""
     arguments = {"apikey": SAB_APIKEY}
     return requests.get("http://%s:%s/%s/" % (host, port, url), params=arguments).text
 
 
 def get_api_result(mode, host=SAB_HOST, port=SAB_PORT, extra_arguments={}):
-    """ Build JSON request to SABnzbd """
+    """Build JSON request to SABnzbd"""
     arguments = {"apikey": SAB_APIKEY, "output": "json", "mode": mode}
     arguments.update(extra_arguments)
     r = requests.get("http://%s:%s/api" % (host, port), params=arguments)
@@ -138,21 +140,26 @@ def get_api_result(mode, host=SAB_HOST, port=SAB_PORT, extra_arguments={}):
     return r.json()
 
 
-def create_nzb(nzb_dir):
-    """ Create NZB from directory using SABNews """
+def create_nzb(nzb_dir: str, metadata: Optional[Dict[str, str]] = None) -> str:
+    """Create NZB from directory using SABNews"""
     nzb_dir_full = os.path.join(SAB_DATA_DIR, nzb_dir)
-    return tests.sabnews.create_nzb(nzb_dir=nzb_dir_full)
+    return tests.sabnews.create_nzb(nzb_dir=nzb_dir_full, metadata=metadata)
 
 
-def create_and_read_nzb(nzbdir):
-    """ Create NZB, return data and delete file """
+def create_and_read_nzb_fp(nzbdir: str, metadata: Optional[Dict[str, str]] = None) -> BinaryIO:
+    """Create NZB, return data and delete file"""
     # Create NZB-file to import
-    nzb_path = create_nzb(nzbdir)
-    with open(nzb_path, "r") as nzb_data_fp:
+    nzb_path = create_nzb(nzbdir, metadata)
+    with open(nzb_path, "rb") as nzb_data_fp:
         nzb_data = nzb_data_fp.read()
     # Remove the created NZB-file
     os.remove(nzb_path)
-    return nzb_data
+    return io.BytesIO(nzb_data)
+
+
+def random_name(lenghth: int = 16) -> str:
+    """Shorthand to create a simple random string"""
+    return "".join(choice(ascii_lowercase + digits) for _ in range(lenghth))
 
 
 class FakeHistoryDB(db.HistoryDB):
@@ -179,15 +186,16 @@ class FakeHistoryDB(db.HistoryDB):
         super().__init__()
 
     def add_fake_history_jobs(self, number_of_entries=1):
-        """ Generate a history db with any number of fake entries """
+        """Generate a history db with any number of fake entries"""
 
         for _ in range(0, number_of_entries):
             nzo = mock.Mock()
 
             # Mock all input build_history_info() needs
             distro_choice = choice(self.distro_names)
-            distro_random = "".join(choice(ascii_lowercase + digits) for i in range(8))
+            distro_random = random_name()
             nzo.password = choice(["secret", ""])
+            nzo.correct_password = "secret"
             nzo.final_name = "%s.%s.Linux.ISO-Usenet" % (distro_choice, distro_random)
             nzo.filename = "%s.%s.Linux-Usenet%s.nzb" % (
                 (distro_choice, distro_random, "{{" + nzo.password + "}}")
@@ -200,27 +208,27 @@ class FakeHistoryDB(db.HistoryDB):
             nzo.status = choice([Status.COMPLETED, choice(self.status_options)])
             nzo.fail_msg = "¡Fracaso absoluto!" if nzo.status == Status.FAILED else ""
             nzo.nzo_id = "SABnzbd_nzo_%s" % ("".join(choice(ascii_lowercase + digits) for i in range(8)))
-            nzo.bytes_downloaded = randint(1024, 1024 ** 4)
+            nzo.bytes_downloaded = randint(1024, 1024**4)
             nzo.md5sum = "".join(choice("abcdef" + digits) for i in range(32))
-            nzo.repair_opts = pp_to_opts(choice(list(db._PP_LOOKUP.keys())))  # for "pp"
-            nzo.nzo_info = {"download_time": randint(1, 10 ** 4)}
+            nzo.repair, nzo.unpack, nzo.delete = pp_to_opts(choice(list(db._PP_LOOKUP.keys())))  # for "pp"
+            nzo.nzo_info = {"download_time": randint(1, 10**4)}
             nzo.unpack_info = {"unpack_info": "placeholder unpack_info line\r\n" * 3}
             nzo.futuretype = False  # for "report", only True when fetching an URL
             nzo.download_path = os.path.join(os.path.dirname(db.HistoryDB.db_path), "placeholder_downpath")
 
             # Mock time when calling add_history_db() to randomize completion times
-            almost_time = mock.Mock(return_value=time.time() - randint(0, 10 ** 8))
+            almost_time = mock.Mock(return_value=time.time() - randint(0, 10**8))
             with mock.patch("time.time", almost_time):
                 self.add_history_db(
                     nzo,
                     storage=os.path.join(os.path.dirname(db.HistoryDB.db_path), "placeholder_workdir"),
-                    postproc_time=randint(1, 10 ** 3),
+                    postproc_time=randint(1, 10**3),
                     script_output="",
                     script_line="",
                 )
 
 
-@pytest.mark.usefixtures("run_sabnzbd_sabnews_and_selenium")
+@pytest.mark.usefixtures("run_sabnzbd", "run_sabnews_and_selenium")
 class SABnzbdBaseTest:
     def no_page_crash(self):
         # Do a base test if CherryPy did not report test
@@ -246,7 +254,7 @@ class SABnzbdBaseTest:
 
     @staticmethod
     def selenium_wrapper(func, *args):
-        """ Wrapper with retries for more stable Selenium """
+        """Wrapper with retries for more stable Selenium"""
         for i in range(3):
             try:
                 return func(*args)
